'use strict';

var ast = require('@eslint-react/ast');
var core = require('@eslint-react/core');
var shared = require('@eslint-react/shared');
var jsx = require('@eslint-react/jsx');
require('string-ts');
var types = require('@typescript-eslint/types');
var _var = require('@eslint-react/var');
var typeUtils = require('@typescript-eslint/type-utils');
var utils = require('@typescript-eslint/utils');
var tsutils = require('ts-api-utils');
var ts = require('typescript');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var tsutils__namespace = /*#__PURE__*/_interopNamespace(tsutils);
var ts__default = /*#__PURE__*/_interopDefault(ts);

var __defProp = Object.defineProperty;
var __export = (target, all3) => {
  for (var name2 in all3)
    __defProp(target, name2, { get: all3[name2], enumerable: true });
};

// package.json
var name = "eslint-plugin-react-core";
var version = "1.5.9-next.2";
var createRule = shared.createRuleForPlugin("core");

// src/utils/get-children-to-array-selector.ts
function getChildrenToArraySelector(reactPragma) {
  return [
    ":matches(",
    "CallExpression",
    `[callee.object.object.name=${reactPragma}]`,
    "[callee.object.property.name=Children]",
    "[callee.property.name=toArray]",
    ",",
    "CallExpression",
    "[callee.object.name=Children]",
    "[callee.property.name=toArray]",
    ")"
  ].join("");
}

// src/rules/ensure-forward-ref-using-ref.ts
var RULE_NAME = "ensure-forward-ref-using-ref";
var ensure_forward_ref_using_ref_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "require all 'forwardRef' components include a 'ref' parameter",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      ENSURE_FORWARD_REF_USING_REF: "'forwardRef' is used with this component but no 'ref' parameter is set."
    },
    schema: []
  },
  name: RULE_NAME,
  create(context) {
    return {
      CallExpression(node) {
        if (!core.isForwardRefCall(node, context))
          return;
        const [component] = node.arguments;
        if (!component || !ast.isFunction(component))
          return;
        const [props, ref] = component.params;
        if (!ref) {
          context.report({
            messageId: "ENSURE_FORWARD_REF_USING_REF",
            node: component
          });
        }
      }
    };
  },
  defaultOptions: []
});

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Function.js
var Function_exports = {};
__export(Function_exports, {
  SK: () => SK,
  absurd: () => absurd,
  apply: () => apply,
  compose: () => compose,
  constFalse: () => constFalse,
  constNull: () => constNull,
  constTrue: () => constTrue,
  constUndefined: () => constUndefined,
  constVoid: () => constVoid,
  constant: () => constant,
  dual: () => dual,
  flip: () => flip,
  flow: () => flow,
  hole: () => hole,
  identity: () => identity,
  isFunction: () => isFunction2,
  pipe: () => pipe,
  satisfies: () => satisfies,
  tupled: () => tupled,
  unsafeCoerce: () => unsafeCoerce,
  untupled: () => untupled
});
var isFunction2 = (input) => typeof input === "function";
var dual = function(arity, body) {
  if (typeof arity === "function") {
    return function() {
      if (arity(arguments)) {
        return body.apply(this, arguments);
      }
      return (self) => body(self, ...arguments);
    };
  }
  switch (arity) {
    case 0:
    case 1:
      throw new RangeError(`Invalid arity ${arity}`);
    case 2:
      return function(a2, b2) {
        if (arguments.length >= 2) {
          return body(a2, b2);
        }
        return function(self) {
          return body(self, a2);
        };
      };
    case 3:
      return function(a2, b2, c2) {
        if (arguments.length >= 3) {
          return body(a2, b2, c2);
        }
        return function(self) {
          return body(self, a2, b2);
        };
      };
    case 4:
      return function(a2, b2, c2, d2) {
        if (arguments.length >= 4) {
          return body(a2, b2, c2, d2);
        }
        return function(self) {
          return body(self, a2, b2, c2);
        };
      };
    case 5:
      return function(a2, b2, c2, d2, e2) {
        if (arguments.length >= 5) {
          return body(a2, b2, c2, d2, e2);
        }
        return function(self) {
          return body(self, a2, b2, c2, d2);
        };
      };
    default:
      return function() {
        if (arguments.length >= arity) {
          return body.apply(this, arguments);
        }
        const args = arguments;
        return function(self) {
          return body(self, ...args);
        };
      };
  }
};
var apply = (a2) => (self) => self(a2);
var identity = (a2) => a2;
var satisfies = () => (b2) => b2;
var unsafeCoerce = identity;
var constant = (value) => () => value;
var constTrue = /* @__PURE__ */ constant(true);
var constFalse = /* @__PURE__ */ constant(false);
var constNull = /* @__PURE__ */ constant(null);
var constUndefined = /* @__PURE__ */ constant(void 0);
var constVoid = constUndefined;
var flip = (f2) => (...b2) => (...a2) => f2(...a2)(...b2);
var compose = /* @__PURE__ */ dual(2, (ab, bc) => (a2) => bc(ab(a2)));
var absurd = (_2) => {
  throw new Error("Called `absurd` function which should be uncallable");
};
var tupled = (f2) => (a2) => f2(...a2);
var untupled = (f2) => (...a2) => f2(a2);
function pipe(a2, ab, bc, cd, de, ef, fg, gh, hi) {
  switch (arguments.length) {
    case 1:
      return a2;
    case 2:
      return ab(a2);
    case 3:
      return bc(ab(a2));
    case 4:
      return cd(bc(ab(a2)));
    case 5:
      return de(cd(bc(ab(a2))));
    case 6:
      return ef(de(cd(bc(ab(a2)))));
    case 7:
      return fg(ef(de(cd(bc(ab(a2))))));
    case 8:
      return gh(fg(ef(de(cd(bc(ab(a2)))))));
    case 9:
      return hi(gh(fg(ef(de(cd(bc(ab(a2))))))));
    default: {
      let ret = arguments[0];
      for (let i2 = 1; i2 < arguments.length; i2++) {
        ret = arguments[i2](ret);
      }
      return ret;
    }
  }
}
function flow(ab, bc, cd, de, ef, fg, gh, hi, ij) {
  switch (arguments.length) {
    case 1:
      return ab;
    case 2:
      return function() {
        return bc(ab.apply(this, arguments));
      };
    case 3:
      return function() {
        return cd(bc(ab.apply(this, arguments)));
      };
    case 4:
      return function() {
        return de(cd(bc(ab.apply(this, arguments))));
      };
    case 5:
      return function() {
        return ef(de(cd(bc(ab.apply(this, arguments)))));
      };
    case 6:
      return function() {
        return fg(ef(de(cd(bc(ab.apply(this, arguments))))));
      };
    case 7:
      return function() {
        return gh(fg(ef(de(cd(bc(ab.apply(this, arguments)))))));
      };
    case 8:
      return function() {
        return hi(gh(fg(ef(de(cd(bc(ab.apply(this, arguments))))))));
      };
    case 9:
      return function() {
        return ij(hi(gh(fg(ef(de(cd(bc(ab.apply(this, arguments)))))))));
      };
  }
  return;
}
var hole = /* @__PURE__ */ unsafeCoerce(absurd);
var SK = (_2, b2) => b2;

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Equivalence.js
var make = (isEquivalent) => (self, that) => self === that || isEquivalent(self, that);

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/internal/version.js
var moduleVersion = "3.0.0";
var getCurrentVersion = () => moduleVersion;

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/GlobalValue.js
var globalStoreId = /* @__PURE__ */ Symbol.for(`effect/GlobalValue/globalStoreId/${/* @__PURE__ */ getCurrentVersion()}`);
if (!(globalStoreId in globalThis)) {
  globalThis[globalStoreId] = /* @__PURE__ */ new Map();
}
var globalStore = globalThis[globalStoreId];
var globalValue = (id, compute) => {
  if (!globalStore.has(id)) {
    globalStore.set(id, compute());
  }
  return globalStore.get(id);
};

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Predicate.js
var Predicate_exports = {};
__export(Predicate_exports, {
  all: () => all,
  and: () => and,
  compose: () => compose2,
  eqv: () => eqv,
  every: () => every,
  hasProperty: () => hasProperty,
  implies: () => implies,
  isBigInt: () => isBigInt,
  isBoolean: () => isBoolean,
  isDate: () => isDate,
  isError: () => isError,
  isFunction: () => isFunction3,
  isIterable: () => isIterable,
  isMap: () => isMap,
  isNever: () => isNever,
  isNotNull: () => isNotNull,
  isNotNullable: () => isNotNullable,
  isNotUndefined: () => isNotUndefined,
  isNull: () => isNull,
  isNullable: () => isNullable,
  isNumber: () => isNumber,
  isObject: () => isObject,
  isPromise: () => isPromise,
  isPromiseLike: () => isPromiseLike,
  isReadonlyRecord: () => isReadonlyRecord,
  isRecord: () => isRecord,
  isSet: () => isSet,
  isString: () => isString,
  isSymbol: () => isSymbol,
  isTagged: () => isTagged,
  isTruthy: () => isTruthy,
  isUint8Array: () => isUint8Array,
  isUndefined: () => isUndefined,
  isUnknown: () => isUnknown,
  mapInput: () => mapInput,
  nand: () => nand,
  nor: () => nor,
  not: () => not,
  or: () => or,
  product: () => product,
  productMany: () => productMany,
  some: () => some,
  struct: () => struct,
  tuple: () => tuple,
  xor: () => xor
});
var mapInput = /* @__PURE__ */ dual(2, (self, f2) => (b2) => self(f2(b2)));
var isTruthy = (input) => !!input;
var isSet = (input) => input instanceof Set;
var isMap = (input) => input instanceof Map;
var isString = (input) => typeof input === "string";
var isNumber = (input) => typeof input === "number";
var isBoolean = (input) => typeof input === "boolean";
var isBigInt = (input) => typeof input === "bigint";
var isSymbol = (input) => typeof input === "symbol";
var isFunction3 = isFunction2;
var isUndefined = (input) => input === void 0;
var isNotUndefined = (input) => input !== void 0;
var isNull = (input) => input === null;
var isNotNull = (input) => input !== null;
var isNever = (_2) => false;
var isUnknown = (_2) => true;
var isRecordOrArray = (input) => typeof input === "object" && input !== null;
var isObject = (input) => isRecordOrArray(input) || isFunction3(input);
var hasProperty = /* @__PURE__ */ dual(2, (self, property) => isObject(self) && property in self);
var isTagged = /* @__PURE__ */ dual(2, (self, tag) => hasProperty(self, "_tag") && self["_tag"] === tag);
var isNullable = (input) => input === null || input === void 0;
var isNotNullable = (input) => input !== null && input !== void 0;
var isError = (input) => input instanceof Error;
var isUint8Array = (input) => input instanceof Uint8Array;
var isDate = (input) => input instanceof Date;
var isIterable = (input) => hasProperty(input, Symbol.iterator);
var isRecord = (input) => isRecordOrArray(input) && !Array.isArray(input);
var isReadonlyRecord = isRecord;
var isPromise = (input) => hasProperty(input, "then") && "catch" in input && isFunction3(input.then) && isFunction3(input.catch);
var isPromiseLike = (input) => hasProperty(input, "then") && isFunction3(input.then);
var compose2 = /* @__PURE__ */ dual(2, (ab, bc) => (a2) => ab(a2) && bc(a2));
var product = (self, that) => ([a2, b2]) => self(a2) && that(b2);
var all = (collection) => {
  return (as2) => {
    let collectionIndex = 0;
    for (const p2 of collection) {
      if (collectionIndex >= as2.length) {
        break;
      }
      if (p2(as2[collectionIndex]) === false) {
        return false;
      }
      collectionIndex++;
    }
    return true;
  };
};
var productMany = (self, collection) => {
  const rest = all(collection);
  return ([head2, ...tail2]) => self(head2) === false ? false : rest(tail2);
};
var tuple = (...elements) => all(elements);
var struct = (fields) => {
  const keys2 = Object.keys(fields);
  return (a2) => {
    for (const key of keys2) {
      if (!fields[key](a2[key])) {
        return false;
      }
    }
    return true;
  };
};
var not = (self) => (a2) => !self(a2);
var or = /* @__PURE__ */ dual(2, (self, that) => (a2) => self(a2) || that(a2));
var and = /* @__PURE__ */ dual(2, (self, that) => (a2) => self(a2) && that(a2));
var xor = /* @__PURE__ */ dual(2, (self, that) => (a2) => self(a2) !== that(a2));
var eqv = /* @__PURE__ */ dual(2, (self, that) => (a2) => self(a2) === that(a2));
var implies = /* @__PURE__ */ dual(2, (self, that) => (a2) => self(a2) ? that(a2) : true);
var nor = /* @__PURE__ */ dual(2, (self, that) => (a2) => !(self(a2) || that(a2)));
var nand = /* @__PURE__ */ dual(2, (self, that) => (a2) => !(self(a2) && that(a2)));
var every = (collection) => (a2) => {
  for (const p2 of collection) {
    if (!p2(a2)) {
      return false;
    }
  }
  return true;
};
var some = (collection) => (a2) => {
  for (const p2 of collection) {
    if (p2(a2)) {
      return true;
    }
  }
  return false;
};

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Utils.js
var GenKindTypeId = /* @__PURE__ */ Symbol.for("effect/Gen/GenKind");
var GenKindImpl = class {
  value;
  constructor(value) {
    this.value = value;
  }
  /**
   * @since 2.0.0
   */
  get _F() {
    return identity;
  }
  /**
   * @since 2.0.0
   */
  get _R() {
    return (_2) => _2;
  }
  /**
   * @since 2.0.0
   */
  get _O() {
    return (_2) => _2;
  }
  /**
   * @since 2.0.0
   */
  get _E() {
    return (_2) => _2;
  }
  /**
   * @since 2.0.0
   */
  [GenKindTypeId] = GenKindTypeId;
  /**
   * @since 2.0.0
   */
  [Symbol.iterator]() {
    return new SingleShotGen(this);
  }
};
var SingleShotGen = class _SingleShotGen {
  self;
  called = false;
  constructor(self) {
    this.self = self;
  }
  /**
   * @since 2.0.0
   */
  next(a2) {
    return this.called ? {
      value: a2,
      done: true
    } : (this.called = true, {
      value: this.self,
      done: false
    });
  }
  /**
   * @since 2.0.0
   */
  return(a2) {
    return {
      value: a2,
      done: true
    };
  }
  /**
   * @since 2.0.0
   */
  throw(e2) {
    throw e2;
  }
  /**
   * @since 2.0.0
   */
  [Symbol.iterator]() {
    return new _SingleShotGen(this.self);
  }
};
var adapter = () => (
  // @ts-expect-error
  function() {
    let x2 = arguments[0];
    for (let i2 = 1; i2 < arguments.length; i2++) {
      x2 = arguments[i2](x2);
    }
    return new GenKindImpl(x2);
  }
);
var defaultIncHi = 335903614;
var defaultIncLo = 4150755663;
var MUL_HI = 1481765933 >>> 0;
var MUL_LO = 1284865837 >>> 0;
var BIT_53 = 9007199254740992;
var BIT_27 = 134217728;
var PCGRandom = class {
  _state;
  constructor(seedHi, seedLo, incHi, incLo) {
    if (isNullable(seedLo) && isNullable(seedHi)) {
      seedLo = Math.random() * 4294967295 >>> 0;
      seedHi = 0;
    } else if (isNullable(seedLo)) {
      seedLo = seedHi;
      seedHi = 0;
    }
    if (isNullable(incLo) && isNullable(incHi)) {
      incLo = this._state ? this._state[3] : defaultIncLo;
      incHi = this._state ? this._state[2] : defaultIncHi;
    } else if (isNullable(incLo)) {
      incLo = incHi;
      incHi = 0;
    }
    this._state = new Int32Array([0, 0, incHi >>> 0, ((incLo || 0) | 1) >>> 0]);
    this._next();
    add64(this._state, this._state[0], this._state[1], seedHi >>> 0, seedLo >>> 0);
    this._next();
    return this;
  }
  /**
   * Returns a copy of the internal state of this random number generator as a
   * JavaScript Array.
   *
   * @category getters
   * @since 2.0.0
   */
  getState() {
    return [this._state[0], this._state[1], this._state[2], this._state[3]];
  }
  /**
   * Restore state previously retrieved using `getState()`.
   *
   * @since 2.0.0
   */
  setState(state) {
    this._state[0] = state[0];
    this._state[1] = state[1];
    this._state[2] = state[2];
    this._state[3] = state[3] | 1;
  }
  /**
   * Get a uniformly distributed 32 bit integer between [0, max).
   *
   * @category getter
   * @since 2.0.0
   */
  integer(max) {
    if (!max) {
      return this._next();
    }
    max = max >>> 0;
    if ((max & max - 1) === 0) {
      return this._next() & max - 1;
    }
    let num = 0;
    const skew = (-max >>> 0) % max >>> 0;
    for (num = this._next(); num < skew; num = this._next()) {
    }
    return num % max;
  }
  /**
   * Get a uniformly distributed IEEE-754 double between 0.0 and 1.0, with
   * 53 bits of precision (every bit of the mantissa is randomized).
   *
   * @category getters
   * @since 2.0.0
   */
  number() {
    const hi = (this._next() & 67108863) * 1;
    const lo = (this._next() & 134217727) * 1;
    return (hi * BIT_27 + lo) / BIT_53;
  }
  /** @internal */
  _next() {
    const oldHi = this._state[0] >>> 0;
    const oldLo = this._state[1] >>> 0;
    mul64(this._state, oldHi, oldLo, MUL_HI, MUL_LO);
    add64(this._state, this._state[0], this._state[1], this._state[2], this._state[3]);
    let xsHi = oldHi >>> 18;
    let xsLo = (oldLo >>> 18 | oldHi << 14) >>> 0;
    xsHi = (xsHi ^ oldHi) >>> 0;
    xsLo = (xsLo ^ oldLo) >>> 0;
    const xorshifted = (xsLo >>> 27 | xsHi << 5) >>> 0;
    const rot = oldHi >>> 27;
    const rot2 = (-rot >>> 0 & 31) >>> 0;
    return (xorshifted >>> rot | xorshifted << rot2) >>> 0;
  }
};
function mul64(out, aHi, aLo, bHi, bLo) {
  let c1 = (aLo >>> 16) * (bLo & 65535) >>> 0;
  let c0 = (aLo & 65535) * (bLo >>> 16) >>> 0;
  let lo = (aLo & 65535) * (bLo & 65535) >>> 0;
  let hi = (aLo >>> 16) * (bLo >>> 16) + ((c0 >>> 16) + (c1 >>> 16)) >>> 0;
  c0 = c0 << 16 >>> 0;
  lo = lo + c0 >>> 0;
  if (lo >>> 0 < c0 >>> 0) {
    hi = hi + 1 >>> 0;
  }
  c1 = c1 << 16 >>> 0;
  lo = lo + c1 >>> 0;
  if (lo >>> 0 < c1 >>> 0) {
    hi = hi + 1 >>> 0;
  }
  hi = hi + Math.imul(aLo, bHi) >>> 0;
  hi = hi + Math.imul(aHi, bLo) >>> 0;
  out[0] = hi;
  out[1] = lo;
}
function add64(out, aHi, aLo, bHi, bLo) {
  let hi = aHi + bHi >>> 0;
  const lo = aLo + bLo >>> 0;
  if (lo >>> 0 < aLo >>> 0) {
    hi = hi + 1 | 0;
  }
  out[0] = hi;
  out[1] = lo;
}

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Hash.js
var randomHashCache = /* @__PURE__ */ globalValue(/* @__PURE__ */ Symbol.for("effect/Hash/randomHashCache"), () => /* @__PURE__ */ new WeakMap());
var pcgr = /* @__PURE__ */ globalValue(/* @__PURE__ */ Symbol.for("effect/Hash/pcgr"), () => new PCGRandom());
var symbol = /* @__PURE__ */ Symbol.for("effect/Hash");
var hash = (self) => {
  switch (typeof self) {
    case "number":
      return number(self);
    case "bigint":
      return string(self.toString(10));
    case "boolean":
      return string(String(self));
    case "symbol":
      return string(String(self));
    case "string":
      return string(self);
    case "undefined":
      return string("undefined");
    case "function":
    case "object": {
      if (self === null) {
        return string("null");
      }
      if (isHash(self)) {
        return self[symbol]();
      } else {
        return random(self);
      }
    }
    default:
      throw new Error(`BUG: unhandled typeof ${typeof self} - please report an issue at https://github.com/Effect-TS/effect/issues`);
  }
};
var random = (self) => {
  if (!randomHashCache.has(self)) {
    randomHashCache.set(self, number(pcgr.integer(Number.MAX_SAFE_INTEGER)));
  }
  return randomHashCache.get(self);
};
var combine = (b2) => (self) => self * 53 ^ b2;
var optimize = (n2) => n2 & 3221225471 | n2 >>> 1 & 1073741824;
var isHash = (u2) => hasProperty(u2, symbol);
var number = (n2) => {
  if (n2 !== n2 || n2 === Infinity) {
    return 0;
  }
  let h2 = n2 | 0;
  if (h2 !== n2) {
    h2 ^= n2 * 4294967295;
  }
  while (n2 > 4294967295) {
    h2 ^= n2 /= 4294967295;
  }
  return optimize(n2);
};
var string = (str) => {
  let h2 = 5381, i2 = str.length;
  while (i2) {
    h2 = h2 * 33 ^ str.charCodeAt(--i2);
  }
  return optimize(h2);
};
var cached = function() {
  if (arguments.length === 1) {
    const self2 = arguments[0];
    return function(hash3) {
      Object.defineProperty(self2, symbol, {
        value() {
          return hash3;
        },
        enumerable: false
      });
      return hash3;
    };
  }
  const self = arguments[0];
  const hash2 = arguments[1];
  Object.defineProperty(self, symbol, {
    value() {
      return hash2;
    },
    enumerable: false
  });
  return hash2;
};

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Equal.js
var symbol2 = /* @__PURE__ */ Symbol.for("effect/Equal");
function equals() {
  if (arguments.length === 1) {
    return (self) => compareBoth(self, arguments[0]);
  }
  return compareBoth(arguments[0], arguments[1]);
}
function compareBoth(self, that) {
  if (self === that) {
    return true;
  }
  const selfType = typeof self;
  if (selfType !== typeof that) {
    return false;
  }
  if ((selfType === "object" || selfType === "function") && self !== null && that !== null) {
    if (isEqual(self) && isEqual(that)) {
      return hash(self) === hash(that) && self[symbol2](that);
    }
  }
  return false;
}
var isEqual = (u2) => hasProperty(u2, symbol2);
var equivalence = () => equals;

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Inspectable.js
var NodeInspectSymbol = /* @__PURE__ */ Symbol.for("nodejs.util.inspect.custom");
var toJSON = (x2) => {
  if (hasProperty(x2, "toJSON") && isFunction3(x2["toJSON"]) && x2["toJSON"].length === 0) {
    return x2.toJSON();
  } else if (Array.isArray(x2)) {
    return x2.map(toJSON);
  }
  return x2;
};
var format = (x2) => JSON.stringify(x2, null, 2);

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Pipeable.js
var pipeArguments = (self, args) => {
  switch (args.length) {
    case 1:
      return args[0](self);
    case 2:
      return args[1](args[0](self));
    case 3:
      return args[2](args[1](args[0](self)));
    case 4:
      return args[3](args[2](args[1](args[0](self))));
    case 5:
      return args[4](args[3](args[2](args[1](args[0](self)))));
    case 6:
      return args[5](args[4](args[3](args[2](args[1](args[0](self))))));
    case 7:
      return args[6](args[5](args[4](args[3](args[2](args[1](args[0](self)))))));
    case 8:
      return args[7](args[6](args[5](args[4](args[3](args[2](args[1](args[0](self))))))));
    case 9:
      return args[8](args[7](args[6](args[5](args[4](args[3](args[2](args[1](args[0](self)))))))));
    default: {
      let ret = self;
      for (let i2 = 0, len = args.length; i2 < len; i2++) {
        ret = args[i2](ret);
      }
      return ret;
    }
  }
};

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/internal/effectable.js
var EffectTypeId = /* @__PURE__ */ Symbol.for("effect/Effect");
var StreamTypeId = /* @__PURE__ */ Symbol.for("effect/Stream");
var SinkTypeId = /* @__PURE__ */ Symbol.for("effect/Sink");
var ChannelTypeId = /* @__PURE__ */ Symbol.for("effect/Channel");
var effectVariance = {
  /* c8 ignore next */
  _R: (_2) => _2,
  /* c8 ignore next */
  _E: (_2) => _2,
  /* c8 ignore next */
  _A: (_2) => _2,
  _V: /* @__PURE__ */ getCurrentVersion()
};
var sinkVariance = {
  /* c8 ignore next */
  _A: (_2) => _2,
  /* c8 ignore next */
  _In: (_2) => _2,
  /* c8 ignore next */
  _L: (_2) => _2,
  /* c8 ignore next */
  _E: (_2) => _2,
  /* c8 ignore next */
  _R: (_2) => _2
};
var channelVariance = {
  /* c8 ignore next */
  _Env: (_2) => _2,
  /* c8 ignore next */
  _InErr: (_2) => _2,
  /* c8 ignore next */
  _InElem: (_2) => _2,
  /* c8 ignore next */
  _InDone: (_2) => _2,
  /* c8 ignore next */
  _OutErr: (_2) => _2,
  /* c8 ignore next */
  _OutElem: (_2) => _2,
  /* c8 ignore next */
  _OutDone: (_2) => _2
};
var EffectPrototype = {
  [EffectTypeId]: effectVariance,
  [StreamTypeId]: effectVariance,
  [SinkTypeId]: sinkVariance,
  [ChannelTypeId]: channelVariance,
  [symbol2](that) {
    return this === that;
  },
  [symbol]() {
    return cached(this, random(this));
  },
  pipe() {
    return pipeArguments(this, arguments);
  }
};

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/internal/option.js
var TypeId = /* @__PURE__ */ Symbol.for("effect/Option");
var CommonProto = {
  ...EffectPrototype,
  [TypeId]: {
    _A: (_2) => _2
  },
  [NodeInspectSymbol]() {
    return this.toJSON();
  },
  toString() {
    return format(this.toJSON());
  }
};
var SomeProto = /* @__PURE__ */ Object.assign(/* @__PURE__ */ Object.create(CommonProto), {
  _tag: "Some",
  _op: "Some",
  [symbol2](that) {
    return isOption(that) && isSome(that) && equals(that.value, this.value);
  },
  [symbol]() {
    return cached(this, combine(hash(this._tag))(hash(this.value)));
  },
  toJSON() {
    return {
      _id: "Option",
      _tag: this._tag,
      value: toJSON(this.value)
    };
  }
});
var NoneHash = /* @__PURE__ */ hash("None");
var NoneProto = /* @__PURE__ */ Object.assign(/* @__PURE__ */ Object.create(CommonProto), {
  _tag: "None",
  _op: "None",
  [symbol2](that) {
    return isOption(that) && isNone(that);
  },
  [symbol]() {
    return NoneHash;
  },
  toJSON() {
    return {
      _id: "Option",
      _tag: this._tag
    };
  }
});
var isOption = (input) => hasProperty(input, TypeId);
var isNone = (fa) => fa._tag === "None";
var isSome = (fa) => fa._tag === "Some";
var none = /* @__PURE__ */ Object.create(NoneProto);
var some2 = (value) => {
  const a2 = Object.create(SomeProto);
  a2.value = value;
  return a2;
};

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/internal/either.js
var TypeId2 = /* @__PURE__ */ Symbol.for("effect/Either");
var CommonProto2 = {
  ...EffectPrototype,
  [TypeId2]: {
    _R: (_2) => _2
  },
  [NodeInspectSymbol]() {
    return this.toJSON();
  },
  toString() {
    return format(this.toJSON());
  }
};
var RightProto = /* @__PURE__ */ Object.assign(/* @__PURE__ */ Object.create(CommonProto2), {
  _tag: "Right",
  _op: "Right",
  [symbol2](that) {
    return isEither(that) && isRight(that) && equals(that.right, this.right);
  },
  [symbol]() {
    return combine(hash(this._tag))(hash(this.right));
  },
  toJSON() {
    return {
      _id: "Either",
      _tag: this._tag,
      right: toJSON(this.right)
    };
  }
});
var LeftProto = /* @__PURE__ */ Object.assign(/* @__PURE__ */ Object.create(CommonProto2), {
  _tag: "Left",
  _op: "Left",
  [symbol2](that) {
    return isEither(that) && isLeft(that) && equals(that.left, this.left);
  },
  [symbol]() {
    return combine(hash(this._tag))(hash(this.left));
  },
  toJSON() {
    return {
      _id: "Either",
      _tag: this._tag,
      left: toJSON(this.left)
    };
  }
});
var isEither = (input) => hasProperty(input, TypeId2);
var isLeft = (ma) => ma._tag === "Left";
var isRight = (ma) => ma._tag === "Right";
var left = (left2) => {
  const a2 = Object.create(LeftProto);
  a2.left = left2;
  return a2;
};
var right = (right2) => {
  const a2 = Object.create(RightProto);
  a2.right = right2;
  return a2;
};
var getLeft = (self) => isRight(self) ? none : some2(self.left);
var getRight = (self) => isLeft(self) ? none : some2(self.right);

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Either.js
var isLeft2 = isLeft;
var isRight2 = isRight;

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Option.js
var Option_exports = {};
__export(Option_exports, {
  Do: () => Do,
  TypeId: () => TypeId3,
  all: () => all2,
  andThen: () => andThen,
  ap: () => ap,
  as: () => as,
  asVoid: () => asVoid,
  bind: () => bind,
  bindTo: () => bindTo,
  composeK: () => composeK,
  contains: () => contains,
  containsWith: () => containsWith,
  exists: () => exists,
  filter: () => filter,
  filterMap: () => filterMap,
  firstSomeOf: () => firstSomeOf,
  flatMap: () => flatMap,
  flatMapNullable: () => flatMapNullable,
  flatten: () => flatten,
  fromIterable: () => fromIterable,
  fromNullable: () => fromNullable,
  gen: () => gen,
  getEquivalence: () => getEquivalence,
  getLeft: () => getLeft2,
  getOrElse: () => getOrElse,
  getOrNull: () => getOrNull,
  getOrThrow: () => getOrThrow,
  getOrThrowWith: () => getOrThrowWith,
  getOrUndefined: () => getOrUndefined,
  getOrder: () => getOrder,
  getRight: () => getRight2,
  isNone: () => isNone2,
  isOption: () => isOption2,
  isSome: () => isSome2,
  let: () => let_,
  lift2: () => lift2,
  liftNullable: () => liftNullable,
  liftPredicate: () => liftPredicate,
  liftThrowable: () => liftThrowable,
  map: () => map,
  match: () => match,
  none: () => none2,
  orElse: () => orElse,
  orElseEither: () => orElseEither,
  orElseSome: () => orElseSome,
  partitionMap: () => partitionMap,
  product: () => product2,
  productMany: () => productMany2,
  reduceCompact: () => reduceCompact,
  some: () => some3,
  tap: () => tap,
  toArray: () => toArray,
  toRefinement: () => toRefinement,
  void: () => void_,
  zipLeft: () => zipLeft,
  zipRight: () => zipRight,
  zipWith: () => zipWith
});

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Order.js
var make2 = (compare) => (self, that) => self === that ? 0 : compare(self, that);

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Option.js
var TypeId3 = /* @__PURE__ */ Symbol.for("effect/Option");
var none2 = () => none;
var some3 = some2;
var isOption2 = isOption;
var isNone2 = isNone;
var isSome2 = isSome;
var match = /* @__PURE__ */ dual(2, (self, {
  onNone,
  onSome
}) => isNone2(self) ? onNone() : onSome(self.value));
var toRefinement = (f2) => (a2) => isSome2(f2(a2));
var fromIterable = (collection) => {
  for (const a2 of collection) {
    return some3(a2);
  }
  return none2();
};
var getRight2 = getRight;
var getLeft2 = getLeft;
var getOrElse = /* @__PURE__ */ dual(2, (self, onNone) => isNone2(self) ? onNone() : self.value);
var orElse = /* @__PURE__ */ dual(2, (self, that) => isNone2(self) ? that() : self);
var orElseSome = /* @__PURE__ */ dual(2, (self, onNone) => isNone2(self) ? some3(onNone()) : self);
var orElseEither = /* @__PURE__ */ dual(2, (self, that) => isNone2(self) ? map(that(), right) : map(self, left));
var firstSomeOf = (collection) => {
  let out = none2();
  for (out of collection) {
    if (isSome2(out)) {
      return out;
    }
  }
  return out;
};
var fromNullable = (nullableValue) => nullableValue == null ? none2() : some3(nullableValue);
var liftNullable = (f2) => (...a2) => fromNullable(f2(...a2));
var getOrNull = /* @__PURE__ */ getOrElse(constNull);
var getOrUndefined = /* @__PURE__ */ getOrElse(constUndefined);
var liftThrowable = (f2) => (...a2) => {
  try {
    return some3(f2(...a2));
  } catch (e2) {
    return none2();
  }
};
var getOrThrowWith = /* @__PURE__ */ dual(2, (self, onNone) => {
  if (isSome2(self)) {
    return self.value;
  }
  throw onNone();
});
var getOrThrow = /* @__PURE__ */ getOrThrowWith(() => new Error("getOrThrow called on a None"));
var map = /* @__PURE__ */ dual(2, (self, f2) => isNone2(self) ? none2() : some3(f2(self.value)));
var as = /* @__PURE__ */ dual(2, (self, b2) => map(self, () => b2));
var asVoid = /* @__PURE__ */ as(void 0);
var void_ = /* @__PURE__ */ some3(void 0);
var flatMap = /* @__PURE__ */ dual(2, (self, f2) => isNone2(self) ? none2() : f2(self.value));
var andThen = /* @__PURE__ */ dual(2, (self, f2) => flatMap(self, (a2) => {
  const b2 = isFunction2(f2) ? f2(a2) : f2;
  return isOption2(b2) ? b2 : some3(b2);
}));
var flatMapNullable = /* @__PURE__ */ dual(2, (self, f2) => isNone2(self) ? none2() : fromNullable(f2(self.value)));
var flatten = /* @__PURE__ */ flatMap(identity);
var zipRight = /* @__PURE__ */ dual(2, (self, that) => flatMap(self, () => that));
var composeK = /* @__PURE__ */ dual(2, (afb, bfc) => (a2) => flatMap(afb(a2), bfc));
var zipLeft = /* @__PURE__ */ dual(2, (self, that) => tap(self, () => that));
var tap = /* @__PURE__ */ dual(2, (self, f2) => flatMap(self, (a2) => map(f2(a2), () => a2)));
var product2 = (self, that) => isSome2(self) && isSome2(that) ? some3([self.value, that.value]) : none2();
var productMany2 = (self, collection) => {
  if (isNone2(self)) {
    return none2();
  }
  const out = [self.value];
  for (const o2 of collection) {
    if (isNone2(o2)) {
      return none2();
    }
    out.push(o2.value);
  }
  return some3(out);
};
var all2 = (input) => {
  if (Symbol.iterator in input) {
    const out2 = [];
    for (const o2 of input) {
      if (isNone2(o2)) {
        return none2();
      }
      out2.push(o2.value);
    }
    return some3(out2);
  }
  const out = {};
  for (const key of Object.keys(input)) {
    const o2 = input[key];
    if (isNone2(o2)) {
      return none2();
    }
    out[key] = o2.value;
  }
  return some3(out);
};
var zipWith = /* @__PURE__ */ dual(3, (self, that, f2) => map(product2(self, that), ([a2, b2]) => f2(a2, b2)));
var ap = /* @__PURE__ */ dual(2, (self, that) => zipWith(self, that, (f2, a2) => f2(a2)));
var reduceCompact = /* @__PURE__ */ dual(3, (self, b2, f2) => {
  let out = b2;
  for (const oa of self) {
    if (isSome2(oa)) {
      out = f2(out, oa.value);
    }
  }
  return out;
});
var toArray = (self) => isNone2(self) ? [] : [self.value];
var partitionMap = /* @__PURE__ */ dual(2, (self, f2) => {
  if (isNone2(self)) {
    return [none2(), none2()];
  }
  const e2 = f2(self.value);
  return isLeft(e2) ? [some3(e2.left), none2()] : [none2(), some3(e2.right)];
});
var filterMap = /* @__PURE__ */ dual(2, (self, f2) => isNone2(self) ? none2() : f2(self.value));
var filter = /* @__PURE__ */ dual(2, (self, predicate) => filterMap(self, (b2) => predicate(b2) ? some2(b2) : none));
var getEquivalence = (isEquivalent) => make((x2, y2) => x2 === y2 || (isNone2(x2) ? isNone2(y2) : isNone2(y2) ? false : isEquivalent(x2.value, y2.value)));
var getOrder = (O2) => make2((self, that) => isSome2(self) ? isSome2(that) ? O2(self.value, that.value) : 1 : -1);
var lift2 = (f2) => dual(2, (self, that) => zipWith(self, that, f2));
var liftPredicate = (predicate) => (b2) => predicate(b2) ? some3(b2) : none2();
var containsWith = (isEquivalent) => dual(2, (self, a2) => isNone2(self) ? false : isEquivalent(self.value, a2));
var _equivalence = /* @__PURE__ */ equivalence();
var contains = /* @__PURE__ */ containsWith(_equivalence);
var exists = /* @__PURE__ */ dual(2, (self, refinement) => isNone2(self) ? false : refinement(self.value));
var bindTo = /* @__PURE__ */ dual(2, (self, name2) => map(self, (a2) => ({
  [name2]: a2
})));
var let_ = /* @__PURE__ */ dual(3, (self, name2, f2) => map(self, (a2) => Object.assign({}, a2, {
  [name2]: f2(a2)
})));
var bind = /* @__PURE__ */ dual(3, (self, name2, f2) => flatMap(self, (a2) => map(f2(a2), (b2) => Object.assign({}, a2, {
  [name2]: b2
}))));
var Do = /* @__PURE__ */ some3({});
var adapter2 = /* @__PURE__ */ adapter();
var gen = (f2) => {
  const iterator = f2(adapter2);
  let state = iterator.next();
  if (state.done) {
    return some3(state.value);
  } else {
    let current = state.value.value;
    if (isNone2(current)) {
      return current;
    }
    while (!state.done) {
      state = iterator.next(current.value);
      if (!state.done) {
        current = state.value.value;
        if (isNone2(current)) {
          return current;
        }
      }
    }
    return some3(state.value);
  }
};

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/Record.js
var Record_exports = {};
__export(Record_exports, {
  collect: () => collect,
  difference: () => difference,
  empty: () => empty,
  every: () => every2,
  filter: () => filter2,
  filterMap: () => filterMap2,
  fromEntries: () => fromEntries,
  fromIterableBy: () => fromIterableBy,
  fromIterableWith: () => fromIterableWith,
  get: () => get,
  getEquivalence: () => getEquivalence2,
  getLefts: () => getLefts,
  getRights: () => getRights,
  getSomes: () => getSomes,
  has: () => has,
  intersection: () => intersection,
  isEmptyReadonlyRecord: () => isEmptyReadonlyRecord,
  isEmptyRecord: () => isEmptyRecord,
  isSubrecord: () => isSubrecord,
  isSubrecordBy: () => isSubrecordBy,
  keys: () => keys,
  map: () => map2,
  mapEntries: () => mapEntries,
  mapKeys: () => mapKeys,
  modify: () => modify,
  modifyOption: () => modifyOption,
  partition: () => partition,
  partitionMap: () => partitionMap2,
  pop: () => pop,
  reduce: () => reduce,
  remove: () => remove,
  replace: () => replace,
  replaceOption: () => replaceOption,
  separate: () => separate,
  set: () => set,
  singleton: () => singleton,
  size: () => size,
  some: () => some4,
  toEntries: () => toEntries,
  union: () => union,
  values: () => values
});
var empty = () => ({});
var isEmptyRecord = (self) => keys(self).length === 0;
var isEmptyReadonlyRecord = isEmptyRecord;
var fromIterableWith = /* @__PURE__ */ dual(2, (self, f2) => {
  const out = empty();
  for (const a2 of self) {
    const [k2, b2] = f2(a2);
    out[k2] = b2;
  }
  return out;
});
var fromIterableBy = (items, f2) => fromIterableWith(items, (a2) => [f2(a2), a2]);
var fromEntries = Object.fromEntries;
var collect = /* @__PURE__ */ dual(2, (self, f2) => {
  const out = [];
  for (const key of keys(self)) {
    out.push(f2(key, self[key]));
  }
  return out;
});
var toEntries = /* @__PURE__ */ collect((key, value) => [key, value]);
var size = (self) => keys(self).length;
var has = /* @__PURE__ */ dual(2, (self, key) => Object.prototype.hasOwnProperty.call(self, key));
var get = /* @__PURE__ */ dual(2, (self, key) => has(self, key) ? some3(self[key]) : none2());
var modify = /* @__PURE__ */ dual(3, (self, key, f2) => {
  if (!has(self, key)) {
    return {
      ...self
    };
  }
  return {
    ...self,
    [key]: f2(self[key])
  };
});
var modifyOption = /* @__PURE__ */ dual(3, (self, key, f2) => {
  if (!has(self, key)) {
    return none2();
  }
  return some3({
    ...self,
    [key]: f2(self[key])
  });
});
var replaceOption = /* @__PURE__ */ dual(3, (self, key, b2) => modifyOption(self, key, () => b2));
var remove = /* @__PURE__ */ dual(2, (self, key) => {
  if (!has(self, key)) {
    return {
      ...self
    };
  }
  const out = {
    ...self
  };
  delete out[key];
  return out;
});
var pop = /* @__PURE__ */ dual(2, (self, key) => has(self, key) ? some3([self[key], remove(self, key)]) : none2());
var map2 = /* @__PURE__ */ dual(2, (self, f2) => {
  const out = {
    ...self
  };
  for (const key of keys(self)) {
    out[key] = f2(self[key], key);
  }
  return out;
});
var mapKeys = /* @__PURE__ */ dual(2, (self, f2) => {
  const out = {};
  for (const key of keys(self)) {
    const a2 = self[key];
    out[f2(key, a2)] = a2;
  }
  return out;
});
var mapEntries = /* @__PURE__ */ dual(2, (self, f2) => {
  const out = {};
  for (const key of keys(self)) {
    const [k2, b2] = f2(self[key], key);
    out[k2] = b2;
  }
  return out;
});
var filterMap2 = /* @__PURE__ */ dual(2, (self, f2) => {
  const out = empty();
  for (const key of keys(self)) {
    const o2 = f2(self[key], key);
    if (isSome2(o2)) {
      out[key] = o2.value;
    }
  }
  return out;
});
var filter2 = /* @__PURE__ */ dual(2, (self, predicate) => {
  const out = empty();
  for (const key of keys(self)) {
    if (predicate(self[key], key)) {
      out[key] = self[key];
    }
  }
  return out;
});
var getSomes = /* @__PURE__ */ filterMap2(identity);
var getLefts = (self) => {
  const out = empty();
  for (const key of keys(self)) {
    const value = self[key];
    if (isLeft2(value)) {
      out[key] = value.left;
    }
  }
  return out;
};
var getRights = (self) => {
  const out = empty();
  for (const key of keys(self)) {
    const value = self[key];
    if (isRight2(value)) {
      out[key] = value.right;
    }
  }
  return out;
};
var partitionMap2 = /* @__PURE__ */ dual(2, (self, f2) => {
  const left2 = empty();
  const right2 = empty();
  for (const key of keys(self)) {
    const e2 = f2(self[key], key);
    if (isLeft2(e2)) {
      left2[key] = e2.left;
    } else {
      right2[key] = e2.right;
    }
  }
  return [left2, right2];
});
var separate = /* @__PURE__ */ partitionMap2(identity);
var partition = /* @__PURE__ */ dual(2, (self, predicate) => {
  const left2 = empty();
  const right2 = empty();
  for (const key of keys(self)) {
    if (predicate(self[key], key)) {
      right2[key] = self[key];
    } else {
      left2[key] = self[key];
    }
  }
  return [left2, right2];
});
var keys = (self) => Object.keys(self);
var values = (self) => collect(self, (_2, a2) => a2);
var set = /* @__PURE__ */ dual(3, (self, key, value) => {
  return {
    ...self,
    [key]: value
  };
});
var replace = /* @__PURE__ */ dual(3, (self, key, value) => {
  if (has(self, key)) {
    return {
      ...self,
      [key]: value
    };
  }
  return {
    ...self
  };
});
var isSubrecordBy = (equivalence2) => dual(2, (self, that) => {
  for (const key of keys(self)) {
    if (!has(that, key) || !equivalence2(self[key], that[key])) {
      return false;
    }
  }
  return true;
});
var isSubrecord = /* @__PURE__ */ isSubrecordBy(/* @__PURE__ */ equivalence());
var reduce = /* @__PURE__ */ dual(3, (self, zero, f2) => {
  let out = zero;
  for (const key of keys(self)) {
    out = f2(out, self[key], key);
  }
  return out;
});
var every2 = /* @__PURE__ */ dual(2, (self, refinement) => {
  for (const key of keys(self)) {
    if (!refinement(self[key], key)) {
      return false;
    }
  }
  return true;
});
var some4 = /* @__PURE__ */ dual(2, (self, predicate) => {
  for (const key of keys(self)) {
    if (predicate(self[key], key)) {
      return true;
    }
  }
  return false;
});
var union = /* @__PURE__ */ dual(3, (self, that, combine2) => {
  if (isEmptyRecord(self)) {
    return {
      ...that
    };
  }
  if (isEmptyRecord(that)) {
    return {
      ...self
    };
  }
  const out = empty();
  for (const key of keys(self)) {
    if (has(that, key)) {
      out[key] = combine2(self[key], that[key]);
    } else {
      out[key] = self[key];
    }
  }
  for (const key of keys(that)) {
    if (!has(out, key)) {
      out[key] = that[key];
    }
  }
  return out;
});
var intersection = /* @__PURE__ */ dual(3, (self, that, combine2) => {
  const out = empty();
  if (isEmptyRecord(self) || isEmptyRecord(that)) {
    return out;
  }
  for (const key of keys(self)) {
    if (has(that, key)) {
      out[key] = combine2(self[key], that[key]);
    }
  }
  return out;
});
var difference = /* @__PURE__ */ dual(2, (self, that) => {
  if (isEmptyRecord(self)) {
    return {
      ...that
    };
  }
  if (isEmptyRecord(that)) {
    return {
      ...self
    };
  }
  const out = {};
  for (const key of keys(self)) {
    if (!has(that, key)) {
      out[key] = self[key];
    }
  }
  for (const key of keys(that)) {
    if (!has(self, key)) {
      out[key] = that[key];
    }
  }
  return out;
});
var getEquivalence2 = (equivalence2) => {
  const is4 = isSubrecordBy(equivalence2);
  return (self, that) => is4(self, that) && is4(that, self);
};
var singleton = (key, value) => ({
  [key]: value
});

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/MutableRef.js
var MutableRef_exports = {};
__export(MutableRef_exports, {
  compareAndSet: () => compareAndSet,
  decrement: () => decrement,
  decrementAndGet: () => decrementAndGet,
  get: () => get2,
  getAndDecrement: () => getAndDecrement,
  getAndIncrement: () => getAndIncrement,
  getAndSet: () => getAndSet,
  getAndUpdate: () => getAndUpdate,
  increment: () => increment,
  incrementAndGet: () => incrementAndGet,
  make: () => make3,
  set: () => set2,
  setAndGet: () => setAndGet,
  toggle: () => toggle,
  update: () => update,
  updateAndGet: () => updateAndGet
});
var TypeId4 = /* @__PURE__ */ Symbol.for("effect/MutableRef");
var MutableRefProto = {
  [TypeId4]: TypeId4,
  toString() {
    return format(this.toJSON());
  },
  toJSON() {
    return {
      _id: "MutableRef",
      current: toJSON(this.current)
    };
  },
  [NodeInspectSymbol]() {
    return this.toJSON();
  },
  pipe() {
    return pipeArguments(this, arguments);
  }
};
var make3 = (value) => {
  const ref = Object.create(MutableRefProto);
  ref.current = value;
  return ref;
};
var compareAndSet = /* @__PURE__ */ dual(3, (self, oldValue, newValue) => {
  if (equals(oldValue, self.current)) {
    self.current = newValue;
    return true;
  }
  return false;
});
var decrement = (self) => update(self, (n2) => n2 - 1);
var decrementAndGet = (self) => updateAndGet(self, (n2) => n2 - 1);
var get2 = (self) => self.current;
var getAndDecrement = (self) => getAndUpdate(self, (n2) => n2 - 1);
var getAndIncrement = (self) => getAndUpdate(self, (n2) => n2 + 1);
var getAndSet = /* @__PURE__ */ dual(2, (self, value) => {
  const ret = self.current;
  self.current = value;
  return ret;
});
var getAndUpdate = /* @__PURE__ */ dual(2, (self, f2) => getAndSet(self, f2(get2(self))));
var increment = (self) => update(self, (n2) => n2 + 1);
var incrementAndGet = (self) => updateAndGet(self, (n2) => n2 + 1);
var set2 = /* @__PURE__ */ dual(2, (self, value) => {
  self.current = value;
  return self;
});
var setAndGet = /* @__PURE__ */ dual(2, (self, value) => {
  self.current = value;
  return self.current;
});
var update = /* @__PURE__ */ dual(2, (self, f2) => set2(self, f2(get2(self))));
var updateAndGet = /* @__PURE__ */ dual(2, (self, f2) => setAndGet(self, f2(get2(self))));
var toggle = (self) => update(self, (_2) => !_2);

// ../../../node_modules/.pnpm/effect@3.0.0/node_modules/effect/dist/esm/MutableList.js
var MutableList_exports = {};
__export(MutableList_exports, {
  append: () => append,
  empty: () => empty2,
  forEach: () => forEach,
  fromIterable: () => fromIterable2,
  head: () => head,
  isEmpty: () => isEmpty,
  length: () => length,
  make: () => make4,
  pop: () => pop2,
  prepend: () => prepend,
  reset: () => reset,
  shift: () => shift,
  tail: () => tail
});
var TypeId5 = /* @__PURE__ */ Symbol.for("effect/MutableList");
var MutableListProto = {
  [TypeId5]: TypeId5,
  [Symbol.iterator]() {
    let done = false;
    let head2 = this.head;
    return {
      next() {
        if (done) {
          return this.return();
        }
        if (head2 == null) {
          done = true;
          return this.return();
        }
        const value = head2.value;
        head2 = head2.next;
        return {
          done,
          value
        };
      },
      return(value) {
        if (!done) {
          done = true;
        }
        return {
          done: true,
          value
        };
      }
    };
  },
  toString() {
    return format(this.toJSON());
  },
  toJSON() {
    return {
      _id: "MutableList",
      values: Array.from(this).map(toJSON)
    };
  },
  [NodeInspectSymbol]() {
    return this.toJSON();
  },
  pipe() {
    return pipeArguments(this, arguments);
  }
};
var makeNode = (value) => ({
  value,
  removed: false,
  prev: void 0,
  next: void 0
});
var empty2 = () => {
  const list = Object.create(MutableListProto);
  list.head = void 0;
  list.tail = void 0;
  list._length = 0;
  return list;
};
var fromIterable2 = (iterable) => {
  const list = empty2();
  for (const element of iterable) {
    append(list, element);
  }
  return list;
};
var make4 = (...elements) => fromIterable2(elements);
var isEmpty = (self) => length(self) === 0;
var length = (self) => self._length;
var tail = (self) => self.tail === void 0 ? void 0 : self.tail.value;
var head = (self) => self.head === void 0 ? void 0 : self.head.value;
var forEach = /* @__PURE__ */ dual(2, (self, f2) => {
  let current = self.head;
  while (current !== void 0) {
    f2(current.value);
    current = current.next;
  }
});
var reset = (self) => {
  self._length = 0;
  self.head = void 0;
  self.tail = void 0;
  return self;
};
var append = /* @__PURE__ */ dual(2, (self, value) => {
  const node = makeNode(value);
  if (self.head === void 0) {
    self.head = node;
  }
  if (self.tail === void 0) {
    self.tail = node;
  } else {
    self.tail.next = node;
    node.prev = self.tail;
    self.tail = node;
  }
  self._length += 1;
  return self;
});
var shift = (self) => {
  const head2 = self.head;
  if (head2 !== void 0) {
    remove2(self, head2);
    return head2.value;
  }
  return void 0;
};
var pop2 = (self) => {
  const tail2 = self.tail;
  if (tail2 !== void 0) {
    remove2(self, tail2);
    return tail2.value;
  }
  return void 0;
};
var prepend = /* @__PURE__ */ dual(2, (self, value) => {
  const node = makeNode(value);
  node.next = self.head;
  if (self.head !== void 0) {
    self.head.prev = node;
  }
  self.head = node;
  if (self.tail === void 0) {
    self.tail = node;
  }
  self._length += 1;
  return self;
});
var remove2 = (self, node) => {
  if (node.removed) {
    return;
  }
  node.removed = true;
  if (node.prev !== void 0 && node.next !== void 0) {
    node.prev.next = node.next;
    node.next.prev = node.prev;
  } else if (node.prev !== void 0) {
    self.tail = node.prev;
    node.prev.next = void 0;
  } else if (node.next !== void 0) {
    self.head = node.next;
    node.next.prev = void 0;
  } else {
    self.tail = void 0;
    self.head = void 0;
  }
  if (self._length > 0) {
    self._length -= 1;
  }
};

// src/rules/no-access-state-in-setstate.ts
var RULE_NAME2 = "no-access-state-in-setstate";
function isKeyLiteralLike(node, property) {
  return property.type === ast.NodeType.Literal || property.type === ast.NodeType.TemplateLiteral && property.expressions.length === 0 || !node.computed && property.type === ast.NodeType.Identifier;
}
function isThisSetState(node) {
  const { callee } = node;
  return callee.type === ast.NodeType.MemberExpression && ast.isThisExpression(callee.object) && callee.property.type === ast.NodeType.Identifier && callee.property.name === "setState";
}
function getName(node) {
  if (node.type === ast.NodeType.TSAsExpression) {
    return getName(node.expression);
  }
  if (node.type === ast.NodeType.Identifier || node.type === ast.NodeType.PrivateIdentifier) {
    return Option_exports.some(node.name);
  }
  if (node.type === ast.NodeType.Literal) {
    return Option_exports.some(String(node.value));
  }
  if (node.type === ast.NodeType.TemplateLiteral && node.expressions.length === 0) {
    return Option_exports.fromNullable(node.quasis[0]?.value.raw);
  }
  return Option_exports.none();
}
var no_access_state_in_setstate_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow accessing 'this.state' within 'setState'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_ACCESS_STATE_IN_SETSTATE: "Do not access 'this.state' within 'setState', use 'setState' callback instead."
    },
    schema: []
  },
  name: RULE_NAME2,
  create(context) {
    const classStack = MutableList_exports.make();
    const methodStack = MutableList_exports.make();
    const setStateStack = MutableList_exports.make();
    return {
      CallExpression(node) {
        if (!isThisSetState(node))
          return;
        MutableList_exports.append(setStateStack, [node, false]);
      },
      "CallExpression:exit"(node) {
        if (!isThisSetState(node))
          return;
        MutableList_exports.pop(setStateStack);
      },
      ClassDeclaration(node) {
        MutableList_exports.append(classStack, [node, core.isClassComponent(node, context)]);
      },
      "ClassDeclaration:exit"() {
        MutableList_exports.pop(classStack);
      },
      ClassExpression(node) {
        MutableList_exports.append(classStack, [node, core.isClassComponent(node, context)]);
      },
      "ClassExpression:exit"() {
        MutableList_exports.pop(classStack);
      },
      MemberExpression(node) {
        if (!ast.isThisExpression(node.object))
          return;
        const [currentClass, isComponent] = MutableList_exports.tail(classStack) ?? [];
        if (!currentClass || !isComponent)
          return;
        const [currentMethod, isStatic] = MutableList_exports.tail(methodStack) ?? [];
        if (!currentMethod || isStatic)
          return;
        const [setState, hasThisState] = MutableList_exports.tail(setStateStack) ?? [];
        if (!setState || hasThisState)
          return;
        if (!Option_exports.exists(getName(node.property), (name2) => name2 === "state"))
          return;
        context.report({
          messageId: "NO_ACCESS_STATE_IN_SETSTATE",
          node
        });
      },
      MethodDefinition(node) {
        MutableList_exports.append(methodStack, [node, node.static]);
      },
      "MethodDefinition:exit"() {
        MutableList_exports.pop(methodStack);
      },
      PropertyDefinition(node) {
        MutableList_exports.append(methodStack, [node, node.static]);
      },
      "PropertyDefinition:exit"() {
        MutableList_exports.pop(methodStack);
      },
      VariableDeclarator(node) {
        const [currentClass, isComponent] = MutableList_exports.tail(classStack) ?? [];
        if (!currentClass || !isComponent)
          return;
        const [currentMethod, isStatic] = MutableList_exports.tail(methodStack) ?? [];
        if (!currentMethod || isStatic)
          return;
        const [setState, hasThisState] = MutableList_exports.tail(setStateStack) ?? [];
        if (!setState || hasThisState)
          return;
        if (!(node.init && ast.isThisExpression(node.init) && node.id.type === ast.NodeType.ObjectPattern))
          return;
        const hasState = node.id.properties.some((prop) => {
          if (prop.type === ast.NodeType.Property && isKeyLiteralLike(prop, prop.key)) {
            return Option_exports.exists(getName(prop.key), (name2) => name2 === "state");
          }
          return false;
        });
        if (!hasState)
          return;
        context.report({
          messageId: "NO_ACCESS_STATE_IN_SETSTATE",
          node
        });
      }
    };
  },
  defaultOptions: []
});

// ../../../node_modules/.pnpm/ts-pattern@5.1.1/node_modules/ts-pattern/dist/index.js
var t = Symbol.for("@ts-pattern/matcher");
var e = Symbol.for("@ts-pattern/isVariadic");
var n = "@ts-pattern/anonymous-select-key";
var r = (t2) => Boolean(t2 && "object" == typeof t2);
var i = (e2) => e2 && !!e2[t];
var o = (n2, s2, c2) => {
  if (i(n2)) {
    const e2 = n2[t](), { matched: r2, selections: i2 } = e2.match(s2);
    return r2 && i2 && Object.keys(i2).forEach((t2) => c2(t2, i2[t2])), r2;
  }
  if (r(n2)) {
    if (!r(s2))
      return false;
    if (Array.isArray(n2)) {
      if (!Array.isArray(s2))
        return false;
      let t2 = [], r2 = [], a2 = [];
      for (const o2 of n2.keys()) {
        const s3 = n2[o2];
        i(s3) && s3[e] ? a2.push(s3) : a2.length ? r2.push(s3) : t2.push(s3);
      }
      if (a2.length) {
        if (a2.length > 1)
          throw new Error("Pattern error: Using `...P.array(...)` several times in a single pattern is not allowed.");
        if (s2.length < t2.length + r2.length)
          return false;
        const e2 = s2.slice(0, t2.length), n3 = 0 === r2.length ? [] : s2.slice(-r2.length), i2 = s2.slice(t2.length, 0 === r2.length ? Infinity : -r2.length);
        return t2.every((t3, n4) => o(t3, e2[n4], c2)) && r2.every((t3, e3) => o(t3, n3[e3], c2)) && (0 === a2.length || o(a2[0], i2, c2));
      }
      return n2.length === s2.length && n2.every((t3, e2) => o(t3, s2[e2], c2));
    }
    return Object.keys(n2).every((e2) => {
      const r2 = n2[e2];
      return (e2 in s2 || i(a2 = r2) && "optional" === a2[t]().matcherType) && o(r2, s2[e2], c2);
      var a2;
    });
  }
  return Object.is(s2, n2);
};
var s = (e2) => {
  var n2, o2, a2;
  return r(e2) ? i(e2) ? null != (n2 = null == (o2 = (a2 = e2[t]()).getSelectionKeys) ? void 0 : o2.call(a2)) ? n2 : [] : Array.isArray(e2) ? c(e2, s) : c(Object.values(e2), s) : [];
};
var c = (t2, e2) => t2.reduce((t3, n2) => t3.concat(e2(n2)), []);
function a(...t2) {
  if (1 === t2.length) {
    const [e2] = t2;
    return (t3) => o(e2, t3, () => {
    });
  }
  if (2 === t2.length) {
    const [e2, n2] = t2;
    return o(e2, n2, () => {
    });
  }
  throw new Error(`isMatching wasn't given the right number of arguments: expected 1 or 2, received ${t2.length}.`);
}
function u(t2) {
  return Object.assign(t2, { optional: () => l(t2), and: (e2) => m(t2, e2), or: (e2) => d(t2, e2), select: (e2) => void 0 === e2 ? p(t2) : p(e2, t2) });
}
function h(t2) {
  return Object.assign(((t3) => Object.assign(t3, { [Symbol.iterator]() {
    let n2 = 0;
    const r2 = [{ value: Object.assign(t3, { [e]: true }), done: false }, { done: true, value: void 0 }];
    return { next: () => {
      var t4;
      return null != (t4 = r2[n2++]) ? t4 : r2.at(-1);
    } };
  } }))(t2), { optional: () => h(l(t2)), select: (e2) => h(void 0 === e2 ? p(t2) : p(e2, t2)) });
}
function l(e2) {
  return u({ [t]: () => ({ match: (t2) => {
    let n2 = {};
    const r2 = (t3, e3) => {
      n2[t3] = e3;
    };
    return void 0 === t2 ? (s(e2).forEach((t3) => r2(t3, void 0)), { matched: true, selections: n2 }) : { matched: o(e2, t2, r2), selections: n2 };
  }, getSelectionKeys: () => s(e2), matcherType: "optional" }) });
}
var f = (t2, e2) => {
  for (const n2 of t2)
    if (!e2(n2))
      return false;
  return true;
};
var g = (t2, e2) => {
  for (const [n2, r2] of t2.entries())
    if (!e2(r2, n2))
      return false;
  return true;
};
function m(...e2) {
  return u({ [t]: () => ({ match: (t2) => {
    let n2 = {};
    const r2 = (t3, e3) => {
      n2[t3] = e3;
    };
    return { matched: e2.every((e3) => o(e3, t2, r2)), selections: n2 };
  }, getSelectionKeys: () => c(e2, s), matcherType: "and" }) });
}
function d(...e2) {
  return u({ [t]: () => ({ match: (t2) => {
    let n2 = {};
    const r2 = (t3, e3) => {
      n2[t3] = e3;
    };
    return c(e2, s).forEach((t3) => r2(t3, void 0)), { matched: e2.some((e3) => o(e3, t2, r2)), selections: n2 };
  }, getSelectionKeys: () => c(e2, s), matcherType: "or" }) });
}
function y(e2) {
  return { [t]: () => ({ match: (t2) => ({ matched: Boolean(e2(t2)) }) }) };
}
function p(...e2) {
  const r2 = "string" == typeof e2[0] ? e2[0] : void 0, i2 = 2 === e2.length ? e2[1] : "string" == typeof e2[0] ? void 0 : e2[0];
  return u({ [t]: () => ({ match: (t2) => {
    let e3 = { [null != r2 ? r2 : n]: t2 };
    return { matched: void 0 === i2 || o(i2, t2, (t3, n2) => {
      e3[t3] = n2;
    }), selections: e3 };
  }, getSelectionKeys: () => [null != r2 ? r2 : n].concat(void 0 === i2 ? [] : s(i2)) }) });
}
function v(t2) {
  return "number" == typeof t2;
}
function b(t2) {
  return "string" == typeof t2;
}
function w(t2) {
  return "bigint" == typeof t2;
}
var S = u(y(function(t2) {
  return true;
}));
var O = S;
var j = (t2) => Object.assign(u(t2), { startsWith: (e2) => {
  return j(m(t2, (n2 = e2, y((t3) => b(t3) && t3.startsWith(n2)))));
  var n2;
}, endsWith: (e2) => {
  return j(m(t2, (n2 = e2, y((t3) => b(t3) && t3.endsWith(n2)))));
  var n2;
}, minLength: (e2) => j(m(t2, ((t3) => y((e3) => b(e3) && e3.length >= t3))(e2))), maxLength: (e2) => j(m(t2, ((t3) => y((e3) => b(e3) && e3.length <= t3))(e2))), includes: (e2) => {
  return j(m(t2, (n2 = e2, y((t3) => b(t3) && t3.includes(n2)))));
  var n2;
}, regex: (e2) => {
  return j(m(t2, (n2 = e2, y((t3) => b(t3) && Boolean(t3.match(n2))))));
  var n2;
} });
var E = j(y(b));
var K = (t2) => Object.assign(u(t2), { between: (e2, n2) => K(m(t2, ((t3, e3) => y((n3) => v(n3) && t3 <= n3 && e3 >= n3))(e2, n2))), lt: (e2) => K(m(t2, ((t3) => y((e3) => v(e3) && e3 < t3))(e2))), gt: (e2) => K(m(t2, ((t3) => y((e3) => v(e3) && e3 > t3))(e2))), lte: (e2) => K(m(t2, ((t3) => y((e3) => v(e3) && e3 <= t3))(e2))), gte: (e2) => K(m(t2, ((t3) => y((e3) => v(e3) && e3 >= t3))(e2))), int: () => K(m(t2, y((t3) => v(t3) && Number.isInteger(t3)))), finite: () => K(m(t2, y((t3) => v(t3) && Number.isFinite(t3)))), positive: () => K(m(t2, y((t3) => v(t3) && t3 > 0))), negative: () => K(m(t2, y((t3) => v(t3) && t3 < 0))) });
var x = K(y(v));
var A = (t2) => Object.assign(u(t2), { between: (e2, n2) => A(m(t2, ((t3, e3) => y((n3) => w(n3) && t3 <= n3 && e3 >= n3))(e2, n2))), lt: (e2) => A(m(t2, ((t3) => y((e3) => w(e3) && e3 < t3))(e2))), gt: (e2) => A(m(t2, ((t3) => y((e3) => w(e3) && e3 > t3))(e2))), lte: (e2) => A(m(t2, ((t3) => y((e3) => w(e3) && e3 <= t3))(e2))), gte: (e2) => A(m(t2, ((t3) => y((e3) => w(e3) && e3 >= t3))(e2))), positive: () => A(m(t2, y((t3) => w(t3) && t3 > 0))), negative: () => A(m(t2, y((t3) => w(t3) && t3 < 0))) });
var P = A(y(w));
var T = u(y(function(t2) {
  return "boolean" == typeof t2;
}));
var k = u(y(function(t2) {
  return "symbol" == typeof t2;
}));
var B = u(y(function(t2) {
  return null == t2;
}));
var _ = u(y(function(t2) {
  return null != t2;
}));
var N = { __proto__: null, matcher: t, optional: l, array: function(...e2) {
  return h({ [t]: () => ({ match: (t2) => {
    if (!Array.isArray(t2))
      return { matched: false };
    if (0 === e2.length)
      return { matched: true };
    const n2 = e2[0];
    let r2 = {};
    if (0 === t2.length)
      return s(n2).forEach((t3) => {
        r2[t3] = [];
      }), { matched: true, selections: r2 };
    const i2 = (t3, e3) => {
      r2[t3] = (r2[t3] || []).concat([e3]);
    };
    return { matched: t2.every((t3) => o(n2, t3, i2)), selections: r2 };
  }, getSelectionKeys: () => 0 === e2.length ? [] : s(e2[0]) }) });
}, set: function(...e2) {
  return u({ [t]: () => ({ match: (t2) => {
    if (!(t2 instanceof Set))
      return { matched: false };
    let n2 = {};
    if (0 === t2.size)
      return { matched: true, selections: n2 };
    if (0 === e2.length)
      return { matched: true };
    const r2 = (t3, e3) => {
      n2[t3] = (n2[t3] || []).concat([e3]);
    }, i2 = e2[0];
    return { matched: f(t2, (t3) => o(i2, t3, r2)), selections: n2 };
  }, getSelectionKeys: () => 0 === e2.length ? [] : s(e2[0]) }) });
}, map: function(...e2) {
  return u({ [t]: () => ({ match: (t2) => {
    if (!(t2 instanceof Map))
      return { matched: false };
    let n2 = {};
    if (0 === t2.size)
      return { matched: true, selections: n2 };
    const r2 = (t3, e3) => {
      n2[t3] = (n2[t3] || []).concat([e3]);
    };
    if (0 === e2.length)
      return { matched: true };
    var i2;
    if (1 === e2.length)
      throw new Error(`\`P.map\` wasn't given enough arguments. Expected (key, value), received ${null == (i2 = e2[0]) ? void 0 : i2.toString()}`);
    const [s2, c2] = e2;
    return { matched: g(t2, (t3, e3) => {
      const n3 = o(s2, e3, r2), i3 = o(c2, t3, r2);
      return n3 && i3;
    }), selections: n2 };
  }, getSelectionKeys: () => 0 === e2.length ? [] : [...s(e2[0]), ...s(e2[1])] }) });
}, intersection: m, union: d, not: function(e2) {
  return u({ [t]: () => ({ match: (t2) => ({ matched: !o(e2, t2, () => {
  }) }), getSelectionKeys: () => [], matcherType: "not" }) });
}, when: y, select: p, any: S, _: O, string: E, number: x, bigint: P, boolean: T, symbol: k, nullish: B, nonNullable: _, instanceOf: function(t2) {
  return u(y(/* @__PURE__ */ function(t3) {
    return (e2) => e2 instanceof t3;
  }(t2)));
}, shape: function(t2) {
  return u(y(a(t2)));
} };
var W = { matched: false, value: void 0 };
function $(t2) {
  return new z(t2, W);
}
var z = class _z {
  constructor(t2, e2) {
    this.input = void 0, this.state = void 0, this.input = t2, this.state = e2;
  }
  with(...t2) {
    if (this.state.matched)
      return this;
    const e2 = t2[t2.length - 1], r2 = [t2[0]];
    let i2;
    3 === t2.length && "function" == typeof t2[1] ? i2 = t2[1] : t2.length > 2 && r2.push(...t2.slice(1, t2.length - 1));
    let s2 = false, c2 = {};
    const a2 = (t3, e3) => {
      s2 = true, c2[t3] = e3;
    }, u2 = !r2.some((t3) => o(t3, this.input, a2)) || i2 && !Boolean(i2(this.input)) ? W : { matched: true, value: e2(s2 ? n in c2 ? c2[n] : c2 : this.input, this.input) };
    return new _z(this.input, u2);
  }
  when(t2, e2) {
    if (this.state.matched)
      return this;
    const n2 = Boolean(t2(this.input));
    return new _z(this.input, n2 ? { matched: true, value: e2(this.input, this.input) } : W);
  }
  otherwise(t2) {
    return this.state.matched ? this.state.value : t2(this.input);
  }
  exhaustive() {
    if (this.state.matched)
      return this.state.value;
    let t2;
    try {
      t2 = JSON.stringify(this.input);
    } catch (e2) {
      t2 = this.input;
    }
    throw new Error(`Pattern matching error: no pattern matches value ${t2}`);
  }
  run() {
    return this.exhaustive();
  }
  returnType() {
    return this;
  }
};

// src/rules/no-array-index-key.ts
var RULE_NAME3 = "no-array-index-key";
var reactChildrenMethod = ["forEach", "map"];
function isReactChildrenMethod(name2) {
  return reactChildrenMethod.some((method) => method === name2);
}
var iteratorFunctionIndexParamPosition = {
  every: 1,
  filter: 1,
  find: 1,
  findIndex: 1,
  findLast: 1,
  findLastIndex: 1,
  flatMap: 1,
  forEach: 1,
  map: 1,
  reduce: 2,
  reduceRight: 2,
  some: 1
};
function isUsingReactChildren(node, context) {
  const { callee } = node;
  if (!("property" in callee) || !("object" in callee) || !("name" in callee.property)) {
    return false;
  }
  if (!isReactChildrenMethod(callee.property.name))
    return false;
  const obj = callee.object;
  if ("name" in obj && obj.name === "Children")
    return true;
  const pragma = jsx.getPragmaFromContext(context);
  return a({ object: { name: pragma } }, obj);
}
function getMapIndexParamName(node, context) {
  const { callee } = node;
  if (callee.type !== ast.NodeType.MemberExpression)
    return Option_exports.none();
  if (callee.property.type !== ast.NodeType.Identifier)
    return Option_exports.none();
  const { name: name2 } = callee.property;
  if (!Record_exports.has(iteratorFunctionIndexParamPosition, name2))
    return Option_exports.none();
  const callbackArg = node.arguments[isUsingReactChildren(node, context) ? 1 : 0];
  if (!callbackArg)
    return Option_exports.none();
  if (!ast.isOneOf([ast.NodeType.ArrowFunctionExpression, ast.NodeType.FunctionExpression])(callbackArg))
    return Option_exports.none();
  const { params } = callbackArg;
  const maybeIndexParamPosition = Record_exports.get(iteratorFunctionIndexParamPosition, name2);
  if (Option_exports.isNone(maybeIndexParamPosition))
    return Option_exports.none();
  const indexParamPosition = maybeIndexParamPosition.value;
  if (params.length < indexParamPosition + 1)
    return Option_exports.none();
  const param = params.at(indexParamPosition);
  return param && "name" in param ? Option_exports.some(param.name) : Option_exports.none();
}
function getIdentifiersFromBinaryExpression(side) {
  if (side.type === ast.NodeType.Identifier)
    return [side];
  if (side.type === ast.NodeType.BinaryExpression) {
    return [
      ...getIdentifiersFromBinaryExpression(side.left),
      ...getIdentifiersFromBinaryExpression(side.right)
    ].filter(Boolean);
  }
  return [];
}
var no_array_index_key_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow using Array index as key",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_ARRAY_INDEX_KEY: "Do not use Array index as key."
    },
    schema: []
  },
  name: RULE_NAME3,
  create(context) {
    const indexParamNames = [];
    function pushIndexParamName(node) {
      Option_exports.map(getMapIndexParamName(node, context), (name2) => indexParamNames.push(name2));
    }
    function popIndexParamName(node) {
      Option_exports.map(getMapIndexParamName(node, context), () => indexParamNames.pop());
    }
    function isArrayIndex(node) {
      return node.type === ast.NodeType.Identifier && indexParamNames.some((name2) => name2 === node.name);
    }
    function checkPropValue(node) {
      if (isArrayIndex(node)) {
        return [{ messageId: "NO_ARRAY_INDEX_KEY", node }];
      }
      if (ast.isOneOf([ast.NodeType.TemplateLiteral, ast.NodeType.BinaryExpression])(node)) {
        const exps = ast.NodeType.TemplateLiteral === node.type ? node.expressions : getIdentifiersFromBinaryExpression(node);
        return exps.reduce((acc, exp) => {
          if (isArrayIndex(exp))
            return [...acc, { messageId: "NO_ARRAY_INDEX_KEY", node: exp }];
          return acc;
        }, []);
      }
      const isToStringCall = a({
        type: ast.NodeType.CallExpression,
        callee: {
          type: ast.NodeType.MemberExpression,
          property: {
            type: ast.NodeType.Identifier,
            name: "toString"
          }
        }
      }, node);
      if (isToStringCall) {
        if (!("object" in node.callee && isArrayIndex(node.callee.object)))
          return [];
        return [{ messageId: "NO_ARRAY_INDEX_KEY", node: node.callee.object }];
      }
      const isStringCall = a({
        type: ast.NodeType.CallExpression,
        callee: {
          type: ast.NodeType.Identifier,
          name: "String"
        }
      }, node);
      if (isStringCall) {
        const [arg] = node.arguments;
        if (arg && isArrayIndex(arg))
          return [{ messageId: "NO_ARRAY_INDEX_KEY", node: arg }];
      }
      return [];
    }
    return {
      CallExpression(node) {
        if ((jsx.isCreateElementCall(node, context) || jsx.isCloneElementCall(node, context)) && node.arguments.length > 1) {
          if (indexParamNames.length === 0)
            return;
          const props = node.arguments[1];
          if (props?.type !== ast.NodeType.ObjectExpression)
            return;
          for (const prop of props.properties) {
            if (!a({ key: { name: "key" } }, prop))
              continue;
            if (!("value" in prop))
              continue;
            const descriptors = checkPropValue(prop.value);
            for (const descriptor of descriptors) {
              context.report(descriptor);
            }
          }
        }
        pushIndexParamName(node);
      },
      "CallExpression:exit": popIndexParamName,
      JSXAttribute(node) {
        if (node.name.name !== "key")
          return;
        if (indexParamNames.length === 0)
          return;
        const { value } = node;
        if (value?.type !== ast.NodeType.JSXExpressionContainer)
          return;
        const descriptors = checkPropValue(value.expression);
        for (const descriptor of descriptors) {
          context.report(descriptor);
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME4 = "no-children-count";
var no_children_count_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'Children.count'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_COUNT: "Using 'Children.count' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME4,
  create(context) {
    const pragma = jsx.getPragmaFromContext(context);
    return {
      MemberExpression(node) {
        if (core.isChildrenCount(node, context, pragma)) {
          context.report({
            messageId: "NO_CHILDREN_COUNT",
            node: node.property
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME5 = "no-children-for-each";
var no_children_for_each_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'Children.forEach'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_FOR_EACH: "Using 'Children.forEach' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME5,
  create(context) {
    const pragma = jsx.getPragmaFromContext(context);
    return {
      MemberExpression(node) {
        if (core.isChildrenForEach(node, context, pragma)) {
          context.report({
            messageId: "NO_CHILDREN_FOR_EACH",
            node: node.property
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME6 = "no-children-map";
var no_children_map_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'Children.map'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_MAP: "Using 'Children.map' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME6,
  create(context) {
    const pragma = jsx.getPragmaFromContext(context);
    return {
      MemberExpression(node) {
        if (core.isChildrenMap(node, context, pragma)) {
          context.report({
            messageId: "NO_CHILDREN_MAP",
            node: node.property
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME7 = "no-children-only";
var no_children_only_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'Children.only'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_ONLY: "Using 'Children.only' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME7,
  create(context) {
    const pragma = jsx.getPragmaFromContext(context);
    return {
      MemberExpression(node) {
        if (core.isChildrenOnly(node, context, pragma)) {
          context.report({
            messageId: "NO_CHILDREN_ONLY",
            node: node.property
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME8 = "no-children-prop";
var no_children_prop_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow passing of 'children' as props",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_PROP: "Children should always be actual children, not passed in as a prop."
    },
    schema: []
  },
  name: RULE_NAME8,
  create(context) {
    return {
      CallExpression(node) {
        if (node.arguments.length === 0)
          return;
        const initialScope = context.sourceCode.getScope(node);
        if (!jsx.isCreateElementCall(node, context))
          return;
        const [_2, props] = node.arguments;
        if (!props || props.type !== ast.NodeType.ObjectExpression)
          return;
        Option_exports.map(jsx.findPropInProperties(props.properties, context, initialScope)("children"), (prop) => {
          context.report({
            messageId: "NO_CHILDREN_PROP",
            node: prop
          });
        });
      },
      JSXElement(node) {
        const initialScope = context.sourceCode.getScope(node);
        Option_exports.map(jsx.getProp(node.openingElement.attributes, "children", context, initialScope), (prop) => {
          context.report({
            messageId: "NO_CHILDREN_PROP",
            node: prop
          });
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME9 = "no-children-to-array";
var no_children_to_array_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'Children.toArray'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CHILDREN_TO_ARRAY: "Using 'Children.toArray' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME9,
  create(context) {
    const pragma = jsx.getPragmaFromContext(context);
    return {
      MemberExpression(node) {
        if (core.isChildrenToArray(node, context, pragma)) {
          context.report({
            messageId: "NO_CHILDREN_TO_ARRAY",
            node: node.property
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME10 = "no-class-component";
var no_class_component_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow class component"
    },
    messages: {
      NO_CLASS_COMPONENT: "Do not use class components. Use function components instead."
    },
    schema: []
  },
  name: RULE_NAME10,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy(context);
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { name: name2, node: component } of components.values()) {
          context.report({
            data: {
              name: Option_exports.getOrElse(Function_exports.constant("anonymous"))(name2)
            },
            messageId: "NO_CLASS_COMPONENT",
            node: component
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME11 = "no-clone-element";
var no_clone_element_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'cloneElement'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CLONE_ELEMENT: "Using 'cloneElement' is uncommon and can lead to fragile code. Use alternatives instead."
    },
    schema: []
  },
  name: RULE_NAME11,
  create(context) {
    return {
      CallExpression(node) {
        if (!jsx.isCloneElementCall(node, context))
          return;
        context.report({
          messageId: "NO_CLONE_ELEMENT",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME12 = "no-comment-textnodes";
var no_comment_textnodes_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow comments from being inserted as text nodes",
      requiresTypeChecking: false
    },
    messages: {
      NO_COMMENT_TEXTNODES: "Possible misused comment in text node. Comments inside children section of tag should be placed inside braces."
    },
    schema: []
  },
  name: RULE_NAME12,
  create(context) {
    function checkText(node) {
      if (ast.isOneOf([types.AST_NODE_TYPES.JSXAttribute, types.AST_NODE_TYPES.JSXExpressionContainer])(node.parent))
        return false;
      const rawValue = context.sourceCode.getText(node);
      return /^\s*\/(\/|\*)/mu.test(rawValue) && node.parent.type.includes("JSX");
    }
    const check = (node) => {
      if (!ast.isOneOf([types.AST_NODE_TYPES.JSXElement, types.AST_NODE_TYPES.JSXFragment])(node.parent))
        return;
      if (!checkText(node))
        return;
      context.report({
        messageId: "NO_COMMENT_TEXTNODES",
        node
      });
    };
    return {
      JSXText: check,
      Literal: check
    };
  },
  defaultOptions: []
});
var RULE_NAME13 = "no-complicated-conditional-rendering";
var no_complicated_conditional_rendering_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow complicated conditional rendering",
      requiresTypeChecking: true
    },
    messages: {
      NO_COMPLICATED_CONDITIONAL_RENDERING: "Avoid complicated conditional rendering. Extract the logic into separate elements or components."
    },
    schema: []
  },
  name: RULE_NAME13,
  create(context) {
    const isConditionalOrLogicalExp = ast.isOneOf([ast.NodeType.ConditionalExpression, ast.NodeType.LogicalExpression]);
    function isComplicatedExpression(node) {
      return $(node).with({ type: ast.NodeType.LogicalExpression }, ({ left: left2, right: right2 }) => {
        return isConditionalOrLogicalExp(left2) || isConditionalOrLogicalExp(right2);
      }).with({ type: ast.NodeType.ConditionalExpression }, ({ alternate, consequent }) => {
        return isConditionalOrLogicalExp(alternate) || isConditionalOrLogicalExp(consequent);
      }).otherwise(Function_exports.constFalse);
    }
    return {
      "JSXExpressionContainer > ConditionalExpression"(node) {
        if (isComplicatedExpression(node)) {
          context.report({
            messageId: "NO_COMPLICATED_CONDITIONAL_RENDERING",
            node
          });
        }
      },
      "JSXExpressionContainer > LogicalExpression"(node) {
        if (isComplicatedExpression(node)) {
          context.report({
            messageId: "NO_COMPLICATED_CONDITIONAL_RENDERING",
            node
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME14 = "no-component-will-mount";
function isComponentWillMount(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentWillMount";
}
var no_component_will_mount_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'componentWillMount'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_COMPONENT_WILL_MOUNT: "'componentWillMount' is deprecated, use 'UNSAFE_componentWillMount' instead."
    },
    schema: []
  },
  name: RULE_NAME14,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy(context);
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isComponentWillMount(member)) {
              context.report({
                messageId: "NO_COMPONENT_WILL_MOUNT",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME15 = "no-component-will-receive-props";
function isComponentWillUpdate(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentWillReceiveProps";
}
var no_component_will_receive_props_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'componentWillReceiveProps'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_COMPONENT_WILL_RECEIVE_PROPS: "'componentWillReceiveProps' is deprecated, use 'UNSAFE_componentWillReceiveProps' instead."
    },
    schema: []
  },
  name: RULE_NAME15,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy(context);
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isComponentWillUpdate(member)) {
              context.report({
                messageId: "NO_COMPONENT_WILL_RECEIVE_PROPS",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME16 = "no-component-will-update";
function isComponentWillUpdate2(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentWillUpdate";
}
var no_component_will_update_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'componentWillUpdate'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_COMPONENT_WILL_UPDATE: "'componentWillUpdate' is deprecated. Use 'UNSAFE_componentWillUpdate' instead."
    },
    schema: []
  },
  name: RULE_NAME16,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy(context);
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isComponentWillUpdate2(member)) {
              context.report({
                messageId: "NO_COMPONENT_WILL_UPDATE",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME17 = "no-create-ref";
var no_create_ref_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'createRef' in function components",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_CREATE_REF: "'createRef' is not allowed in function components. Use 'useRef' instead."
    },
    schema: []
  },
  name: RULE_NAME17,
  create(context) {
    const { ctx, listeners } = core.useComponentCollector(context);
    const possibleCreateRefCalls = /* @__PURE__ */ new Map();
    return {
      ...listeners,
      CallExpression(node) {
        if (!core.isCreateRefCall(node, context))
          return;
        Option_exports.map(
          ctx.getCurrentFunction(),
          ([currentFn]) => possibleCreateRefCalls.set(currentFn, [...possibleCreateRefCalls.get(currentFn) ?? [], node])
        );
      },
      "Program:exit"(node) {
        const components = Array.from(ctx.getAllComponents(node).values());
        for (const { node: component } of components) {
          const createRefCalls = possibleCreateRefCalls.get(component);
          if (!createRefCalls)
            continue;
          for (const createRefCall of createRefCalls) {
            context.report({
              messageId: "NO_CREATE_REF",
              node: createRefCall
            });
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME18 = "no-direct-mutation-state";
function getName2(node) {
  if (node.type === ast.NodeType.TSAsExpression) {
    return getName2(node.expression);
  }
  if (node.type === ast.NodeType.Identifier || node.type === ast.NodeType.PrivateIdentifier) {
    return Option_exports.some(node.name);
  }
  if (node.type === ast.NodeType.Literal) {
    return Option_exports.some(String(node.value));
  }
  if (node.type === ast.NodeType.TemplateLiteral && node.expressions.length === 0) {
    return Option_exports.fromNullable(node.quasis[0]?.value.raw);
  }
  return Option_exports.none();
}
function isAssignmentToThisState(node) {
  const { left: left2 } = node;
  return left2.type === ast.NodeType.MemberExpression && ast.isThisExpression(left2.object) && Option_exports.exists(getName2(left2.property), (name2) => name2 === "state");
}
function isConstructorFunction(node) {
  return ast.isOneOf([ast.NodeType.FunctionDeclaration, ast.NodeType.FunctionExpression])(node) && ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node.parent) && node.parent.key.type === ast.NodeType.Identifier && node.parent.key.name === "constructor" && node.parent.parent.type === ast.NodeType.ClassBody;
}
var no_direct_mutation_state_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow direct mutation of state",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_DIRECT_MUTATION_STATE: "Do not mutate state directly. Use 'setState()' instead."
    },
    schema: []
  },
  name: RULE_NAME18,
  create(context) {
    return {
      AssignmentExpression(node) {
        if (!isAssignmentToThisState(node))
          return;
        const maybeParentClass = ast.traverseUpGuard(node, ast.isOneOf([ast.NodeType.ClassDeclaration, ast.NodeType.ClassExpression]));
        if (Option_exports.isNone(maybeParentClass))
          return;
        const parentClass = maybeParentClass.value;
        if (!core.isClassComponent(parentClass, context))
          return;
        const maybeParentConstructor = ast.traverseUpGuard(node, isConstructorFunction);
        if (Option_exports.exists(maybeParentConstructor, (n2) => context.sourceCode.getScope(node).block === n2))
          return;
        context.report({
          messageId: "NO_DIRECT_MUTATION_STATE",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME19 = "no-duplicate-key";
var no_duplicate_key_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow duplicate keys in 'key' prop when rendering list",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_DUPLICATE_KEY: "Duplicate key '{{value}}' found."
    },
    schema: []
  },
  name: RULE_NAME19,
  create(context) {
    const reactPragma = jsx.getPragmaFromContext(context);
    const childrenToArraySelector = getChildrenToArraySelector(reactPragma);
    const isWithinChildrenToArrayRef = MutableRef_exports.make(false);
    function checkIteratorElement(node) {
      if (node.type !== ast.NodeType.JSXElement)
        return Option_exports.none();
      const initialScope = context.sourceCode.getScope(node);
      return Function_exports.pipe(
        jsx.findPropInAttributes(node.openingElement.attributes, context, initialScope)("key"),
        Option_exports.flatMap((k2) => "value" in k2 ? Option_exports.fromNullable(k2.value) : Option_exports.none()),
        Option_exports.flatMap((v2) => {
          return ast.isNodeEqual(v2, v2) ? Option_exports.some({
            data: {
              value: context.sourceCode.getText(v2)
            },
            messageId: "NO_DUPLICATE_KEY",
            node: v2
          }) : Option_exports.none();
        })
      );
    }
    function checkExpression(node) {
      return $(node).with({ type: ast.NodeType.JSXElement }, checkIteratorElement).with({ type: ast.NodeType.JSXFragment }, checkIteratorElement).with({ type: ast.NodeType.ConditionalExpression }, (n2) => {
        if (!("consequent" in n2))
          return Option_exports.none();
        return Option_exports.orElse(checkIteratorElement(n2.consequent), () => checkIteratorElement(n2.alternate));
      }).with({ type: ast.NodeType.LogicalExpression }, (n2) => {
        if (!("left" in n2))
          return Option_exports.none();
        return Option_exports.orElse(checkIteratorElement(n2.left), () => checkIteratorElement(n2.right));
      }).otherwise(Option_exports.none);
    }
    function checkBlockStatement(node) {
      return ast.getNestedReturnStatements(node).reduce((acc, statement) => {
        if (!statement.argument)
          return acc;
        const maybeDescriptor = checkIteratorElement(statement.argument);
        if (Option_exports.isNone(maybeDescriptor))
          return acc;
        const descriptor = maybeDescriptor.value;
        return [...acc, descriptor];
      }, []);
    }
    const seen = /* @__PURE__ */ new WeakSet();
    return {
      [`${childrenToArraySelector}:exit`]() {
        MutableRef_exports.set(isWithinChildrenToArrayRef, false);
      },
      "ArrayExpression, JSXElement > JSXElement"(node) {
        if (MutableRef_exports.get(isWithinChildrenToArrayRef))
          return;
        const elements = $(node).with({ type: ast.NodeType.ArrayExpression }, ({ elements: elements2 }) => elements2).with({ type: ast.NodeType.JSXElement }, ({ parent }) => "children" in parent ? parent.children : []).otherwise(() => []).filter(ast.is(ast.NodeType.JSXElement)).filter((element) => !seen.has(element));
        const keys2 = elements.reduce(
          (acc, element) => {
            const attr = element.openingElement.attributes.findLast(a({
              type: ast.NodeType.JSXAttribute,
              name: {
                name: "key"
              }
            }));
            if (!attr || !("value" in attr) || attr.value === null)
              return acc;
            const { value } = attr;
            if (acc.length === 0)
              return [[element, attr, value]];
            if (acc.some(([_2, _1, v2]) => ast.isNodeEqual(v2, value))) {
              return [...acc, [element, attr, value]];
            }
            return acc;
          },
          []
        );
        if (keys2.length < 2)
          return;
        for (const [element, attr, value] of keys2) {
          seen.add(element);
          context.report({
            data: {
              value: context.sourceCode.getText(value)
            },
            messageId: "NO_DUPLICATE_KEY",
            node: attr
          });
        }
      },
      CallExpression(node) {
        const isMapCall = a({
          callee: {
            type: ast.NodeType.MemberExpression,
            property: {
              name: "map"
            }
          }
        }, node);
        const isArrayFromCall = a({
          type: ast.NodeType.CallExpression,
          callee: {
            type: ast.NodeType.MemberExpression,
            property: {
              name: "from"
            }
          }
        }, node);
        if (!isMapCall && !isArrayFromCall)
          return;
        if (MutableRef_exports.get(isWithinChildrenToArrayRef))
          return;
        const fn = node.arguments[isMapCall ? 0 : 1];
        if (!ast.isOneOf([ast.NodeType.ArrowFunctionExpression, ast.NodeType.FunctionExpression])(fn))
          return;
        if (fn.body.type === ast.NodeType.BlockStatement) {
          for (const descriptor of checkBlockStatement(fn.body)) {
            context.report(descriptor);
          }
          return;
        }
        Option_exports.map(checkExpression(fn.body), context.report);
      },
      [childrenToArraySelector]() {
        MutableRef_exports.set(isWithinChildrenToArrayRef, true);
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME20 = "no-implicit-key";
var no_implicit_key_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow spreading 'key' from objects.",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_IMPLICIT_KEY: "Prefer specifying key explicitly instead of spreading it from object."
    },
    schema: []
  },
  name: RULE_NAME20,
  create(context) {
    function check(node) {
      const initialScope = context.sourceCode.getScope(node);
      return Function_exports.pipe(
        jsx.findPropInAttributes(node.attributes, context, initialScope)("key"),
        Option_exports.filter(ast.is(ast.NodeType.JSXSpreadAttribute)),
        Option_exports.map((key) => ({
          messageId: "NO_IMPLICIT_KEY",
          node: key
        }))
      );
    }
    return {
      JSXOpeningElement: Function_exports.flow(check, Option_exports.map(context.report))
    };
  },
  defaultOptions: []
});
var RULE_NAME21 = "no-leaked-conditional-rendering";
var allowedVariants = [
  "nullish",
  "boolean",
  "string",
  "falsy string",
  "truthy boolean",
  "truthy string"
];
function inspectVariantTypes(types) {
  const variantTypes = /* @__PURE__ */ new Set();
  if (types.some((type) => tsutils__namespace.isTypeFlagSet(type, ts__default.default.TypeFlags.Null | ts__default.default.TypeFlags.Undefined | ts__default.default.TypeFlags.VoidLike))) {
    variantTypes.add("nullish");
  }
  const booleans = types.filter((type) => tsutils__namespace.isTypeFlagSet(type, ts__default.default.TypeFlags.BooleanLike));
  switch (true) {
    case (booleans.length === 1 && !!booleans[0]): {
      const [first] = booleans;
      Function_exports.pipe(
        $(first).when(tsutils__namespace.isTrueLiteralType, () => Option_exports.some("truthy boolean")).when(tsutils__namespace.isFalseLiteralType, () => Option_exports.some("falsy boolean")).otherwise(Option_exports.none),
        Option_exports.map((v2) => variantTypes.add(v2))
      );
      break;
    }
    case booleans.length === 2: {
      variantTypes.add("boolean");
      break;
    }
  }
  const strings = types.filter((type) => tsutils__namespace.isTypeFlagSet(type, ts__default.default.TypeFlags.StringLike));
  if (strings.length > 0) {
    const evaluated = $(strings).when(
      (types2) => types2.every((type) => type.isStringLiteral() && type.value !== ""),
      Function_exports.constant("truthy string")
    ).when(
      (types2) => types2.every((type) => type.isStringLiteral() && type.value === ""),
      Function_exports.constant("falsy string")
    ).otherwise(Function_exports.constant("string"));
    variantTypes.add(evaluated);
  }
  const numbers = types.filter(
    (type) => tsutils__namespace.isTypeFlagSet(
      type,
      ts__default.default.TypeFlags.NumberLike | ts__default.default.TypeFlags.BigIntLike
    )
  );
  if (numbers.length > 0) {
    const evaluated = $(numbers).when(
      (types2) => types2.every((type) => type.isNumberLiteral() && type.value !== 0),
      Function_exports.constant("truthy number")
    ).when(
      (types2) => types2.every((type) => type.isNumberLiteral() && type.value === 0),
      Function_exports.constant("falsy number")
    ).otherwise(Function_exports.constant("number"));
    variantTypes.add(evaluated);
  }
  if (types.some((type) => tsutils__namespace.isTypeFlagSet(type, ts__default.default.TypeFlags.EnumLike))) {
    variantTypes.add("enum");
  }
  if (types.some(
    (type) => !tsutils__namespace.isTypeFlagSet(
      type,
      ts__default.default.TypeFlags.Null | ts__default.default.TypeFlags.Undefined | ts__default.default.TypeFlags.VoidLike | ts__default.default.TypeFlags.BooleanLike | ts__default.default.TypeFlags.StringLike | ts__default.default.TypeFlags.NumberLike | ts__default.default.TypeFlags.BigIntLike | ts__default.default.TypeFlags.TypeParameter | ts__default.default.TypeFlags.Any | ts__default.default.TypeFlags.Unknown | ts__default.default.TypeFlags.Never
    )
  )) {
    variantTypes.add("object");
  }
  if (types.some(
    (type) => tsutils__namespace.isTypeFlagSet(
      type,
      ts__default.default.TypeFlags.TypeParameter | ts__default.default.TypeFlags.Any | ts__default.default.TypeFlags.Unknown
    )
  )) {
    variantTypes.add("any");
  }
  if (types.some((type) => tsutils__namespace.isTypeFlagSet(type, ts__default.default.TypeFlags.Never))) {
    variantTypes.add("never");
  }
  return variantTypes;
}
var no_leaked_conditional_rendering_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow problematic leaked values from being rendered",
      recommended: "recommended",
      requiresTypeChecking: true
    },
    messages: {
      NO_LEAKED_CONDITIONAL_RENDERING: "Potential leaked value that might cause unintentionally rendered values or rendering crashes"
    },
    schema: []
  },
  name: RULE_NAME21,
  create(context) {
    const services = utils.ESLintUtils.getParserServices(context);
    function checkExpression(node) {
      return $(node).when(ast.isJSX, Option_exports.none).with({ type: ast.NodeType.LogicalExpression, operator: "&&" }, ({ left: left2, right: right2 }) => {
        const isLeftUnaryNot = a({
          type: ast.NodeType.UnaryExpression,
          operator: "!"
        }, left2);
        if (isLeftUnaryNot)
          return checkExpression(right2);
        const leftType = typeUtils.getConstrainedTypeAtLocation(services, left2);
        const leftTypeVariants = inspectVariantTypes(tsutils__namespace.unionTypeParts(leftType));
        const isLeftValid = Array.from(leftTypeVariants.values()).every((type) => allowedVariants.some((allowed) => allowed === type));
        if (isLeftValid)
          return checkExpression(right2);
        return Option_exports.some({
          messageId: "NO_LEAKED_CONDITIONAL_RENDERING",
          node: left2
        });
      }).with({ type: ast.NodeType.ConditionalExpression }, ({ alternate, consequent }) => {
        return Option_exports.orElse(checkExpression(consequent), () => checkExpression(alternate));
      }).with({ type: ast.NodeType.Identifier }, (n2) => {
        const initialScope = context.sourceCode.getScope(n2);
        return Function_exports.pipe(
          _var.findVariable(n2.name, initialScope),
          Option_exports.flatMap(_var.getVariableInitExpression(0)),
          Option_exports.flatMap(checkExpression)
        );
      }).otherwise(Option_exports.none);
    }
    return {
      "JSXExpressionContainer > ConditionalExpression"(node) {
        Option_exports.map(checkExpression(node), context.report);
      },
      "JSXExpressionContainer > LogicalExpression"(node) {
        Option_exports.map(checkExpression(node), context.report);
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME22 = "no-missing-component-display-name";
var no_missing_component_display_name_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "require 'displayName' for memo and forwardRef components",
      requiresTypeChecking: false
    },
    messages: {
      NO_MISSING_COMPONENT_DISPLAY_NAME: "Missing 'displayName' for component."
    },
    schema: []
  },
  name: RULE_NAME22,
  create(context) {
    const { ctx, listeners } = core.useComponentCollector(context);
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { displayName, flag, node: node2 } of components.values()) {
          const isMemoOrForwardRef = Boolean(flag & core.ERFunctionComponentFlag.ForwardRef) || Boolean(flag & core.ERFunctionComponentFlag.Memo);
          if (Option_exports.isSome(ast.getFunctionIdentifier(node2)))
            continue;
          if (!isMemoOrForwardRef)
            continue;
          if (Option_exports.isNone(displayName)) {
            context.report({
              messageId: "NO_MISSING_COMPONENT_DISPLAY_NAME",
              node: node2
            });
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME23 = "no-missing-key";
var no_missing_key_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "require 'key' prop when rendering list",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_MISSING_KEY: "Missing 'key' prop for element when rendering list",
      NO_MISSING_KEY_WITH_FRAGMENT: "Use '{{reactPragma}}.{{fragmentPragma}}' component instead of '<>' because it does not support key prop"
    },
    schema: []
  },
  name: RULE_NAME23,
  create(context) {
    const reactPragma = jsx.getPragmaFromContext(context);
    const fragmentPragma = jsx.getFragmentFromContext(context);
    const childrenToArraySelector = getChildrenToArraySelector(reactPragma);
    const isWithinChildrenToArrayRef = MutableRef_exports.make(false);
    function checkIteratorElement(node) {
      const initialScope = context.sourceCode.getScope(node);
      if (node.type === ast.NodeType.JSXElement && !jsx.hasProp(node.openingElement.attributes, "key", context, initialScope)) {
        return Option_exports.some({
          messageId: "NO_MISSING_KEY",
          node
        });
      }
      if (node.type === ast.NodeType.JSXFragment) {
        return Option_exports.some({
          data: {
            fragmentPragma,
            reactPragma
          },
          messageId: "NO_MISSING_KEY_WITH_FRAGMENT",
          node
        });
      }
      return Option_exports.none();
    }
    function checkExpression(node) {
      return $(node).with({ type: ast.NodeType.JSXElement }, checkIteratorElement).with({ type: ast.NodeType.JSXFragment }, checkIteratorElement).with({ type: ast.NodeType.ConditionalExpression }, (n2) => {
        if (!("consequent" in n2))
          return Option_exports.none();
        return Option_exports.orElse(checkIteratorElement(n2.consequent), () => checkIteratorElement(n2.alternate));
      }).with({ type: ast.NodeType.LogicalExpression }, (n2) => {
        if (!("left" in n2))
          return Option_exports.none();
        return Option_exports.orElse(checkIteratorElement(n2.left), () => checkIteratorElement(n2.right));
      }).otherwise(Option_exports.none);
    }
    function checkBlockStatement(node) {
      return ast.getNestedReturnStatements(node).reduce((acc, statement) => {
        if (!statement.argument)
          return acc;
        const maybeDescriptor = checkIteratorElement(statement.argument);
        if (Option_exports.isNone(maybeDescriptor))
          return acc;
        const descriptor = maybeDescriptor.value;
        return [...acc, descriptor];
      }, []);
    }
    return {
      [`${childrenToArraySelector}:exit`]() {
        MutableRef_exports.set(isWithinChildrenToArrayRef, false);
      },
      ArrayExpression(node) {
        if (MutableRef_exports.get(isWithinChildrenToArrayRef))
          return;
        const elements = node.elements.filter(ast.is(ast.NodeType.JSXElement));
        if (elements.length === 0)
          return;
        const initialScope = context.sourceCode.getScope(node);
        for (const element of elements) {
          if (!jsx.hasProp(element.openingElement.attributes, "key", context, initialScope)) {
            context.report({
              messageId: "NO_MISSING_KEY",
              node: element
            });
          }
        }
      },
      CallExpression(node) {
        const isMapCall = a({
          callee: {
            type: ast.NodeType.MemberExpression,
            property: {
              name: "map"
            }
          }
        }, node);
        const isArrayFromCall = a({
          type: ast.NodeType.CallExpression,
          callee: {
            type: ast.NodeType.MemberExpression,
            property: {
              name: "from"
            }
          }
        }, node);
        if (!isMapCall && !isArrayFromCall)
          return;
        if (MutableRef_exports.get(isWithinChildrenToArrayRef))
          return;
        const fn = node.arguments[isMapCall ? 0 : 1];
        if (!ast.isOneOf([ast.NodeType.ArrowFunctionExpression, ast.NodeType.FunctionExpression])(fn))
          return;
        if (fn.body.type === ast.NodeType.BlockStatement) {
          for (const descriptor of checkBlockStatement(fn.body)) {
            context.report(descriptor);
          }
          return;
        }
        Option_exports.map(checkExpression(fn.body), context.report);
      },
      JSXFragment(node) {
        if (MutableRef_exports.get(isWithinChildrenToArrayRef))
          return;
        if (node.parent.type === ast.NodeType.ArrayExpression) {
          context.report({
            data: {
              fragmentPragma,
              reactPragma
            },
            messageId: "NO_MISSING_KEY_WITH_FRAGMENT",
            node
          });
        }
      },
      [childrenToArraySelector]() {
        MutableRef_exports.set(isWithinChildrenToArrayRef, true);
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME24 = "no-nested-components";
var no_nested_components_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of unstable nested components",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NESTED_COMPONENT: "Do not nest components inside other components. Move it to the top level.",
      NESTED_COMPONENT_IN_PROPS: "Do not nest components inside props. Move it to the top level or pass it as a prop."
    },
    schema: []
  },
  name: RULE_NAME24,
  create(context) {
    const hint = core.ERComponentHint.SkipMapCallback | core.ERComponentHint.SkipNullLiteral | core.ERComponentHint.SkipUndefinedLiteral | core.ERComponentHint.SkipBooleanLiteral | core.ERComponentHint.SkipStringLiteral | core.ERComponentHint.SkipNumberLiteral | core.ERComponentHint.StrictLogical | core.ERComponentHint.StrictConditional;
    const collector = core.useComponentCollector(context, hint);
    const collectorLegacy = core.useComponentCollectorLegacy(context);
    return {
      ...collector.listeners,
      ...collectorLegacy.listeners,
      "Program:exit"(node) {
        const functionComponents = Array.from(collector.ctx.getAllComponents(node).values());
        const classComponents = Array.from(collectorLegacy.ctx.getAllComponents(node).values());
        const isFunctionComponent = (node2) => {
          return ast.isFunction(node2) && functionComponents.some((component) => component.node === node2);
        };
        const isClassComponent8 = (node2) => {
          return ast.isClass(node2) && classComponents.some((component) => component.node === node2);
        };
        for (const { name: componentName, node: component } of functionComponents) {
          if (core.unsafeIsDirectValueOfRenderProperty(component))
            continue;
          if (Option_exports.isNone(componentName))
            continue;
          const name2 = componentName.value;
          const isInsideProperty = component.parent.type === ast.NodeType.Property;
          const isInsideJSXPropValue = jsx.isInsidePropValue(component);
          if (isInsideJSXPropValue) {
            if (!core.unsafeIsDeclaredInRenderProp(component)) {
              context.report({
                data: {
                  name: name2
                },
                messageId: "NESTED_COMPONENT_IN_PROPS",
                node: component
              });
            }
            continue;
          }
          if (jsx.isInsideCreateElementProps(component, context)) {
            context.report({
              data: {
                name: name2
              },
              messageId: "NESTED_COMPONENT_IN_PROPS",
              node: component
            });
            continue;
          }
          const maybeParentComponent = ast.traverseUpGuard(component, isFunctionComponent);
          if (Option_exports.isSome(maybeParentComponent) && !core.unsafeIsDirectValueOfRenderProperty(maybeParentComponent.value)) {
            context.report({
              data: {
                name: name2
              },
              messageId: isInsideProperty ? "NESTED_COMPONENT_IN_PROPS" : "NESTED_COMPONENT",
              node: component
            });
            continue;
          }
          const isInsideClassComponentRenderMethod = core.isInsideRenderMethod(component, context);
          if (isInsideClassComponentRenderMethod) {
            context.report({
              data: {
                name: name2
              },
              messageId: "NESTED_COMPONENT",
              node: component
            });
          }
        }
        for (const { name: name2, node: component } of classComponents) {
          if (Option_exports.isNone(ast.traverseUp(component, (node2) => isClassComponent8(node2) || isFunctionComponent(node2))))
            continue;
          context.report({
            data: {
              name: Option_exports.getOrElse(() => "unknown")(name2)
            },
            messageId: "NESTED_COMPONENT",
            node: component
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME25 = "no-redundant-should-component-update";
function isShouldComponentUpdate(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "shouldComponentUpdate";
}
var no_redundant_should_component_update_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'shouldComponentUpdate' in class component extends 'React.PureComponent'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_REDUNDANT_SHOULD_COMPONENT_UPDATE: "'{{componentName}}' does not need 'shouldComponentUpdate' when extending 'React.PureComponent'."
    },
    schema: []
  },
  name: RULE_NAME25,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy(context);
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { name: name2, flag, node: component } of components.values()) {
          if (!(flag & core.ERClassComponentFlag.PureComponent))
            continue;
          const { body } = component.body;
          for (const member of body) {
            if (isShouldComponentUpdate(member)) {
              context.report({
                data: {
                  componentName: Option_exports.getOrElse(() => "PureComponent")(name2)
                },
                messageId: "NO_REDUNDANT_SHOULD_COMPONENT_UPDATE",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME26 = "no-set-state-in-component-did-mount";
function isThisSetState2(node) {
  const { callee } = node;
  return callee.type === ast.NodeType.MemberExpression && callee.object.type === ast.NodeType.ThisExpression && callee.property.type === ast.NodeType.Identifier && callee.property.name === "setState";
}
function isComponentDidMount(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentDidMount";
}
var no_set_state_in_component_did_mount_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'setState' in 'componentDidMount'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_SET_STATE_IN_COMPONENT_DID_MOUNT: "Do not call `this.setState` in `componentDidMount` outside of functions, such as callbacks."
    },
    schema: []
  },
  name: RULE_NAME26,
  create(context) {
    return {
      CallExpression(node) {
        if (!isThisSetState2(node))
          return;
        const maybeParentClass = ast.traverseUp(node, ast.isOneOf([ast.NodeType.ClassDeclaration, ast.NodeType.ClassExpression]));
        if (Option_exports.isNone(maybeParentClass))
          return;
        const parentClass = maybeParentClass.value;
        if (!core.isClassComponent(parentClass, context))
          return;
        const maybeParentMethod = ast.traverseUp(node, isComponentDidMount);
        if (Option_exports.isNone(maybeParentMethod))
          return;
        const parentMethod = maybeParentMethod.value;
        if (parentMethod.parent !== parentClass.body)
          return;
        if (context.sourceCode.getScope(node).upper !== context.sourceCode.getScope(parentMethod))
          return;
        context.report({
          messageId: "NO_SET_STATE_IN_COMPONENT_DID_MOUNT",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME27 = "no-set-state-in-component-did-update";
function isThisSetState3(node) {
  const { callee } = node;
  return callee.type === ast.NodeType.MemberExpression && callee.object.type === ast.NodeType.ThisExpression && callee.property.type === ast.NodeType.Identifier && callee.property.name === "setState";
}
function isComponentDidUpdate(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentDidUpdate";
}
var no_set_state_in_component_did_update_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'setState' in 'componentDidUpdate'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_SET_STATE_IN_COMPONENT_DID_UPDATE: "Do not call `this.setState` in `componentDidUpdate` outside of functions, such as callbacks."
    },
    schema: []
  },
  name: RULE_NAME27,
  create(context) {
    return {
      CallExpression(node) {
        if (!isThisSetState3(node))
          return;
        const maybeParentClass = ast.traverseUp(node, ast.isOneOf([ast.NodeType.ClassDeclaration, ast.NodeType.ClassExpression]));
        if (Option_exports.isNone(maybeParentClass))
          return;
        const parentClass = maybeParentClass.value;
        if (!core.isClassComponent(parentClass, context))
          return;
        const maybeParentMethod = ast.traverseUp(node, isComponentDidUpdate);
        if (Option_exports.isNone(maybeParentMethod))
          return;
        const parentMethod = maybeParentMethod.value;
        if (parentMethod.parent !== parentClass.body)
          return;
        if (context.sourceCode.getScope(node).upper !== context.sourceCode.getScope(parentMethod))
          return;
        context.report({
          messageId: "NO_SET_STATE_IN_COMPONENT_DID_UPDATE",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME28 = "no-set-state-in-component-will-update";
function isThisSetState4(node) {
  const { callee } = node;
  return callee.type === ast.NodeType.MemberExpression && callee.object.type === ast.NodeType.ThisExpression && callee.property.type === ast.NodeType.Identifier && callee.property.name === "setState";
}
function isComponentWillUpdate3(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "componentWillUpdate";
}
var no_set_state_in_component_will_update_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow 'setState' in 'componentWillUpdate'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_SET_STATE_IN_COMPONENT_WILL_UPDATE: "Do not call `this.setState` in `componentWillUpdate` outside of functions, such as callbacks."
    },
    schema: []
  },
  name: RULE_NAME28,
  create(context) {
    return {
      CallExpression(node) {
        if (!isThisSetState4(node))
          return;
        const maybeParentClass = ast.traverseUp(node, ast.isOneOf([ast.NodeType.ClassDeclaration, ast.NodeType.ClassExpression]));
        if (Option_exports.isNone(maybeParentClass))
          return;
        const parentClass = maybeParentClass.value;
        if (!core.isClassComponent(parentClass, context))
          return;
        const maybeParentMethod = ast.traverseUp(node, isComponentWillUpdate3);
        if (Option_exports.isNone(maybeParentMethod))
          return;
        const parentMethod = maybeParentMethod.value;
        if (parentMethod.parent !== parentClass.body)
          return;
        if (context.sourceCode.getScope(node).upper !== context.sourceCode.getScope(parentMethod))
          return;
        context.report({
          messageId: "NO_SET_STATE_IN_COMPONENT_WILL_UPDATE",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME29 = "no-string-refs";
function containsStringLiteral({ value }) {
  return value?.type === ast.NodeType.Literal && Predicate_exports.isString(value.value);
}
function containsStringExpressionContainer({ value }) {
  if (value?.type !== ast.NodeType.JSXExpressionContainer)
    return false;
  if (value.expression.type === ast.NodeType.Literal)
    return Predicate_exports.isString(value.expression.value);
  return value.expression.type === ast.NodeType.TemplateLiteral;
}
var no_string_refs_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow using deprecated string refs",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_STRING_REFS: "The string refs are deprecated, use callback refs instead."
    },
    schema: []
  },
  name: RULE_NAME29,
  create(context) {
    return {
      JSXAttribute(node) {
        if (node.name.name !== "ref")
          return;
        if (containsStringLiteral(node) || containsStringExpressionContainer(node)) {
          context.report({
            messageId: "NO_STRING_REFS",
            node
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME30 = "no-unsafe-component-will-mount";
function isUnsafeComponentWillMount(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "UNSAFE_componentWillMount";
}
var no_unsafe_component_will_mount_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'UNSAFE_componentWillMount'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNSAFE_COMPONENT_WILL_MOUNT: "Do not use 'UNSAFE_componentWillMount'."
    },
    schema: []
  },
  name: RULE_NAME30,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy(context);
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isUnsafeComponentWillMount(member)) {
              context.report({
                messageId: "NO_UNSAFE_COMPONENT_WILL_MOUNT",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME31 = "no-unsafe-component-will-receive-props";
function isUnsafeComponentWillReceiveProps(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "UNSAFE_componentWillReceiveProps";
}
var no_unsafe_component_will_receive_props_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'UNSAFE_componentWillReceiveProps'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNSAFE_COMPONENT_WILL_RECEIVE_PROPS: "Do not use 'UNSAFE_componentWillReceiveProps'."
    },
    schema: []
  },
  name: RULE_NAME31,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy(context);
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isUnsafeComponentWillReceiveProps(member)) {
              context.report({
                messageId: "NO_UNSAFE_COMPONENT_WILL_RECEIVE_PROPS",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME32 = "no-unsafe-component-will-update";
function isUnsafeComponentWillMount2(node) {
  return ast.isOneOf([ast.NodeType.MethodDefinition, ast.NodeType.PropertyDefinition])(node) && node.key.type === ast.NodeType.Identifier && node.key.name === "UNSAFE_componentWillUpdate";
}
var no_unsafe_component_will_update_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of 'UNSAFE_componentWillUpdate'",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNSAFE_COMPONENT_WILL_UPDATE: "Do not use 'UNSAFE_componentWillUpdate'."
    },
    schema: []
  },
  name: RULE_NAME32,
  create(context) {
    const { ctx, listeners } = core.useComponentCollectorLegacy(context);
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { body } = component.body;
          for (const member of body) {
            if (isUnsafeComponentWillMount2(member)) {
              context.report({
                messageId: "NO_UNSAFE_COMPONENT_WILL_UPDATE",
                node: member
              });
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME33 = "no-unstable-context-value";
var no_unstable_context_value_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow passing constructed values to context providers",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNSTABLE_CONTEXT_VALUE: "The '{{type}}' passed as the value prop to the context provider should not be constructed. It will change on every render.",
      NO_UNSTABLE_CONTEXT_VALUE_WITH_FUNCTION: "The '{{type}}' passed as the value prop to the context provider should not be constructed. It will change on every render. Consider wrapping it in a useCallback hook.",
      NO_UNSTABLE_CONTEXT_VALUE_WITH_IDENTIFIER: "The '{{type}}' passed as the value prop to the context provider should not be constructed. It will change on every render. Consider wrapping it in a useMemo hook."
    },
    schema: []
  },
  name: RULE_NAME33,
  create(context) {
    const { ctx, listeners } = core.useComponentCollector(context);
    const possibleValueConstructions = /* @__PURE__ */ new Map();
    return {
      ...listeners,
      JSXOpeningElement(node) {
        const openingElementName = node.name;
        if (openingElementName.type !== ast.NodeType.JSXMemberExpression)
          return;
        if (openingElementName.property.name !== "Provider")
          return;
        const maybeJSXValueAttribute = Option_exports.fromNullable(
          node.attributes.find((attribute) => {
            return attribute.type === ast.NodeType.JSXAttribute && attribute.name.name === "value";
          })
        );
        if (Option_exports.isNone(maybeJSXValueAttribute) || !("value" in maybeJSXValueAttribute.value))
          return;
        const valueNode = maybeJSXValueAttribute.value.value;
        if (valueNode?.type !== ast.NodeType.JSXExpressionContainer)
          return;
        const valueExpression = valueNode.expression;
        const construction = ast.inspectConstruction(valueExpression, context);
        if (construction._tag === "None")
          return;
        Option_exports.map(
          ctx.getCurrentFunction(),
          ([currentFn]) => possibleValueConstructions.set(currentFn, [
            ...possibleValueConstructions.get(currentFn) ?? [],
            construction
          ])
        );
      },
      "Program:exit"(node) {
        const components = Array.from(ctx.getAllComponents(node).values());
        for (const { node: component } of components) {
          const constructions = possibleValueConstructions.get(component);
          if (!constructions)
            continue;
          for (const construction of constructions) {
            if (construction._tag === "None")
              continue;
            const { _tag, node: constructionNode } = construction;
            const messageId = _tag.startsWith("Function") ? "NO_UNSTABLE_CONTEXT_VALUE_WITH_FUNCTION" : "NO_UNSTABLE_CONTEXT_VALUE_WITH_IDENTIFIER";
            context.report({
              data: {
                type: ast.readableNodeType(constructionNode)
              },
              messageId,
              node: constructionNode
            });
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME34 = "no-unstable-default-props";
var no_unstable_default_props_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow usage of unstable value as default param in function component",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNSTABLE_DEFAULT_PROPS: "found a/an '{{forbiddenType}}' as default prop. This could lead to potential infinite render loop in React. Use a variable instead of '{{forbiddenType}}'."
    },
    schema: []
  },
  name: RULE_NAME34,
  create(context) {
    const { ctx, listeners } = core.useComponentCollector(context);
    const possibleDestructuringDeclarators = /* @__PURE__ */ new WeakMap();
    return {
      ...listeners,
      "Program:exit"(node) {
        const components = ctx.getAllComponents(node);
        for (const { node: component } of components.values()) {
          const { params } = component;
          const [props] = params;
          if (!props)
            continue;
          const properties = $(props).with({ type: ast.NodeType.ObjectPattern }, ({ properties: properties2 }) => properties2).with({ type: ast.NodeType.Identifier }, ({ name: name2 }) => {
            const variableDeclarators = possibleDestructuringDeclarators.get(component);
            if (!variableDeclarators)
              return [];
            const declarators = variableDeclarators.filter((d2) => d2.init.name === name2);
            return declarators.flatMap((d2) => d2.id.properties);
          }).otherwise(() => []);
          for (const prop of properties) {
            if (prop.type !== ast.NodeType.Property || prop.value.type !== ast.NodeType.AssignmentPattern)
              continue;
            const { value } = prop;
            const { right: right2 } = value;
            const construction = ast.inspectConstruction(value, context, ast.ConstructionHint.StrictCallExpression);
            if (construction._tag === "None")
              continue;
            const forbiddenType = ast.readableNodeType(right2);
            context.report({
              data: {
                forbiddenType
              },
              messageId: "NO_UNSTABLE_DEFAULT_PROPS",
              node: right2
            });
          }
        }
      },
      "VariableDeclarator[id.type='ObjectPattern'][init.type='Identifier']"(node) {
        Option_exports.map(
          ctx.getCurrentFunction(),
          ([currentFn]) => possibleDestructuringDeclarators.set(currentFn, [
            ...possibleDestructuringDeclarators.get(currentFn) ?? [],
            node
          ])
        );
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME35 = "no-unused-class-component-members";
var LIFECYCLE_METHODS = /* @__PURE__ */ new Set([
  "constructor",
  "componentDidCatch",
  "componentDidMount",
  "componentDidUpdate",
  "componentWillMount",
  "componentWillReceiveProps",
  "componentWillUnmount",
  "componentWillUpdate",
  "getSnapshotBeforeUpdate",
  "shouldComponentUpdate",
  "UNSAFE_componentWillMount",
  "UNSAFE_componentWillReceiveProps",
  "UNSAFE_componentWillUpdate",
  "state",
  "render"
]);
function isKeyLiteralLike2(node, property) {
  return property.type === ast.NodeType.Literal || property.type === ast.NodeType.TemplateLiteral && property.expressions.length === 0 || !node.computed && property.type === ast.NodeType.Identifier;
}
function getName3(node) {
  if (node.type === ast.NodeType.TSAsExpression) {
    return getName3(node.expression);
  }
  if (node.type === ast.NodeType.Identifier || node.type === ast.NodeType.PrivateIdentifier) {
    return Option_exports.some(node.name);
  }
  if (node.type === ast.NodeType.Literal) {
    return Option_exports.some(String(node.value));
  }
  if (node.type === ast.NodeType.TemplateLiteral && node.expressions.length === 0) {
    return Option_exports.fromNullable(node.quasis[0]?.value.raw);
  }
  return Option_exports.none();
}
var no_unused_class_component_members_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow unused class component members",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNUSED_CLASS_COMPONENT_MEMBERS: "Unused method or property '{{methodName}}'' of class '{{className}}'."
    },
    schema: []
  },
  name: RULE_NAME35,
  create(context) {
    const classStack = MutableList_exports.make();
    const methodStack = MutableList_exports.make();
    const propertyDefs = /* @__PURE__ */ new WeakMap();
    const propertyUsages = /* @__PURE__ */ new WeakMap();
    function classEnter(node) {
      MutableList_exports.append(classStack, node);
      if (!core.isClassComponent(node, context))
        return;
      propertyDefs.set(node, /* @__PURE__ */ new Set());
      propertyUsages.set(node, /* @__PURE__ */ new Set());
    }
    function classExit() {
      const currentClass = MutableList_exports.pop(classStack);
      if (!currentClass || !core.isClassComponent(currentClass, context))
        return;
      const className = Option_exports.map(ast.getClassIdentifier(currentClass), (id) => id.name);
      const defs = propertyDefs.get(currentClass);
      const usages = propertyUsages.get(currentClass);
      if (!defs)
        return;
      for (const def of defs) {
        const name2 = getName3(def);
        if (Option_exports.isNone(name2))
          continue;
        if (!!usages?.has(name2.value) || LIFECYCLE_METHODS.has(name2.value))
          continue;
        context.report({
          data: {
            className: Option_exports.getOrElse(className, () => "Component"),
            methodName: name2.value
          },
          messageId: "NO_UNUSED_CLASS_COMPONENT_MEMBERS",
          node: def
        });
      }
    }
    function methodEnter(node) {
      MutableList_exports.append(methodStack, node);
      const currentClass = MutableList_exports.tail(classStack);
      if (!currentClass || !core.isClassComponent(currentClass, context))
        return;
      if (node.static)
        return;
      if (isKeyLiteralLike2(node, node.key)) {
        propertyDefs.get(currentClass)?.add(node.key);
      }
    }
    function methodExit() {
      MutableList_exports.pop(methodStack);
    }
    return {
      ClassDeclaration: classEnter,
      "ClassDeclaration:exit": classExit,
      ClassExpression: classEnter,
      "ClassExpression:exit": classExit,
      MemberExpression(node) {
        const currentClass = MutableList_exports.tail(classStack);
        const currentMethod = MutableList_exports.tail(methodStack);
        if (!currentClass || !currentMethod)
          return;
        if (!core.isClassComponent(currentClass, context) || currentMethod.static)
          return;
        if (!ast.isThisExpression(node.object) || !isKeyLiteralLike2(node, node.property))
          return;
        if (node.parent.type === ast.NodeType.AssignmentExpression && node.parent.left === node) {
          propertyDefs.get(currentClass)?.add(node.property);
          return;
        }
        Option_exports.map(getName3(node.property), (name2) => propertyUsages.get(currentClass)?.add(name2));
      },
      MethodDefinition: methodEnter,
      "MethodDefinition:exit": methodExit,
      PropertyDefinition: methodEnter,
      "PropertyDefinition:exit": methodExit,
      VariableDeclarator(node) {
        const currentClass = MutableList_exports.tail(classStack);
        const currentMethod = MutableList_exports.tail(methodStack);
        if (!currentClass || !currentMethod)
          return;
        if (!core.isClassComponent(currentClass, context) || currentMethod.static)
          return;
        if (node.init && ast.isThisExpression(node.init) && node.id.type === ast.NodeType.ObjectPattern) {
          for (const prop of node.id.properties) {
            if (prop.type === ast.NodeType.Property && isKeyLiteralLike2(prop, prop.key)) {
              Option_exports.map(getName3(prop.key), (name2) => propertyUsages.get(currentClass)?.add(name2));
            }
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME36 = "no-unused-state";
function isKeyLiteralLike3(node, property) {
  return property.type === ast.NodeType.Literal || property.type === ast.NodeType.TemplateLiteral && property.expressions.length === 0 || !node.computed && property.type === ast.NodeType.Identifier;
}
function getName4(node) {
  if (node.type === ast.NodeType.TSAsExpression) {
    return getName4(node.expression);
  }
  if (node.type === ast.NodeType.Identifier || node.type === ast.NodeType.PrivateIdentifier) {
    return Option_exports.some(node.name);
  }
  if (node.type === ast.NodeType.Literal) {
    return Option_exports.some(String(node.value));
  }
  if (node.type === ast.NodeType.TemplateLiteral && node.expressions.length === 0) {
    return Option_exports.fromNullable(node.quasis[0]?.value.raw);
  }
  return Option_exports.none();
}
function isAssignmentToThisState2(node) {
  const { left: left2 } = node;
  return left2.type === ast.NodeType.MemberExpression && ast.isThisExpression(left2.object) && Option_exports.exists(getName4(left2.property), (name2) => name2 === "state");
}
var isGetDerivedStateFromProps = a({
  key: {
    name: "getDerivedStateFromProps"
  },
  static: true,
  value: {
    params: N.array()
  }
});
var no_unused_state_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "Prevents unused state of class component.",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_UNUSED_STATE: "Unused class component state."
    },
    schema: []
  },
  name: RULE_NAME36,
  create(context) {
    const classStack = MutableList_exports.make();
    const methodStack = MutableList_exports.make();
    const constructorStack = MutableList_exports.make();
    const stateDefs = /* @__PURE__ */ new WeakMap();
    function classEnter(node) {
      MutableList_exports.append(classStack, node);
    }
    function classExit() {
      const currentClass = MutableList_exports.pop(classStack);
      if (!currentClass || !core.isClassComponent(currentClass, context))
        return;
      const className = Option_exports.map(ast.getClassIdentifier(currentClass), (id) => id.name);
      const [def, isUsed] = stateDefs.get(currentClass) ?? [Option_exports.none(), false];
      if (Option_exports.isNone(def) || isUsed)
        return;
      context.report({
        data: {
          className: Option_exports.getOrElse(className, () => "Component")
        },
        messageId: "NO_UNUSED_STATE",
        node: def.value
      });
    }
    function methodEnter(node) {
      MutableList_exports.append(methodStack, node);
      const currentClass = MutableList_exports.tail(classStack);
      if (!currentClass || !core.isClassComponent(currentClass, context))
        return;
      if (node.static) {
        if (isGetDerivedStateFromProps(node) && node.value.params.length > 1) {
          const [def] = stateDefs.get(currentClass) ?? [Option_exports.none()];
          stateDefs.set(currentClass, [def, true]);
        }
        return;
      }
      if (Option_exports.exists(getName4(node.key), (name2) => name2 === "state")) {
        stateDefs.set(currentClass, [Option_exports.some(node.key), false]);
      }
    }
    function methodExit() {
      MutableList_exports.pop(methodStack);
    }
    function constructorEnter(node) {
      MutableList_exports.append(constructorStack, node);
    }
    function constructorExit() {
      MutableList_exports.pop(constructorStack);
    }
    return {
      AssignmentExpression(node) {
        if (!isAssignmentToThisState2(node))
          return;
        const currentClass = MutableList_exports.tail(classStack);
        if (!currentClass || !core.isClassComponent(currentClass, context))
          return;
        const currentConstructor = MutableList_exports.tail(constructorStack);
        if (!currentConstructor || !currentClass.body.body.includes(currentConstructor))
          return;
        const [_2, isUsed] = stateDefs.get(currentClass) ?? [Option_exports.none(), false];
        stateDefs.set(currentClass, [Option_exports.some(node.left), isUsed]);
      },
      ClassDeclaration: classEnter,
      "ClassDeclaration:exit": classExit,
      ClassExpression: classEnter,
      "ClassExpression:exit": classExit,
      MemberExpression(node) {
        if (!ast.isThisExpression(node.object))
          return;
        if (!Option_exports.exists(getName4(node.property), (name2) => name2 === "state"))
          return;
        const currentClass = MutableList_exports.tail(classStack);
        if (!currentClass || !core.isClassComponent(currentClass, context))
          return;
        const currentMethod = MutableList_exports.tail(methodStack);
        if (!currentMethod || currentMethod.static)
          return;
        if (currentMethod === MutableList_exports.tail(constructorStack))
          return;
        if (!currentClass.body.body.includes(currentMethod))
          return;
        const [def] = stateDefs.get(currentClass) ?? [Option_exports.none(), false];
        stateDefs.set(currentClass, [def, true]);
      },
      MethodDefinition: methodEnter,
      "MethodDefinition:exit": methodExit,
      "MethodDefinition[key.name='constructor']": constructorEnter,
      "MethodDefinition[key.name='constructor']:exit": constructorExit,
      PropertyDefinition: methodEnter,
      "PropertyDefinition:exit": methodExit,
      VariableDeclarator(node) {
        const currentClass = MutableList_exports.tail(classStack);
        if (!currentClass || !core.isClassComponent(currentClass, context))
          return;
        const currentMethod = MutableList_exports.tail(methodStack);
        if (!currentMethod || currentMethod.static)
          return;
        if (currentMethod === MutableList_exports.tail(constructorStack))
          return;
        if (!currentClass.body.body.includes(currentMethod))
          return;
        if (!(node.init && ast.isThisExpression(node.init) && node.id.type === ast.NodeType.ObjectPattern))
          return;
        const hasState = node.id.properties.some((prop) => {
          if (prop.type === ast.NodeType.Property && isKeyLiteralLike3(prop, prop.key)) {
            return Option_exports.exists(getName4(prop.key), (name2) => name2 === "state");
          }
          return false;
        });
        if (!hasState)
          return;
        const [def] = stateDefs.get(currentClass) ?? [Option_exports.none(), false];
        stateDefs.set(currentClass, [def, true]);
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME37 = "no-useless-fragment";
function isFragmentWithOnlyTextAndIsNotChild(node) {
  return node.children.length === 1 && jsx.isLiteral(node.children[0]) && !(node.parent.type === ast.NodeType.JSXElement || node.parent.type === ast.NodeType.JSXFragment);
}
function containsCallExpression(node) {
  return node.type === ast.NodeType.JSXExpressionContainer && node.expression.type === ast.NodeType.CallExpression;
}
function isFragmentHasLessThanTwoChildren(node) {
  const nonPaddingChildren = node.children.filter(
    (child) => !jsx.isPaddingSpaces(child)
  );
  if (nonPaddingChildren.length === 1 && nonPaddingChildren[0]) {
    return !containsCallExpression(nonPaddingChildren[0]);
  }
  return nonPaddingChildren.length === 0;
}
function isFragmentWithSingleExpression(node) {
  const children = node.children.filter((child) => !jsx.isPaddingSpaces(child));
  return children.length === 1 && children[0]?.type === ast.NodeType.JSXExpressionContainer;
}
var no_useless_fragment_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow unnecessary fragments",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      NO_USELESS_FRAGMENT: "Fragments containing a single element are usually unnecessary.",
      NO_USELESS_FRAGMENT_IN_BUILT_IN: "Passing a fragment to a built-in component is unnecessary."
    },
    schema: []
  },
  name: RULE_NAME37,
  create(context) {
    const reactPragma = jsx.getPragmaFromContext(context);
    const fragmentPragma = jsx.getFragmentFromContext(context);
    function checkNode(node) {
      const initialScope = context.sourceCode.getScope(node);
      if (node.type === ast.NodeType.JSXElement && jsx.hasProp(
        node.openingElement.attributes,
        "key",
        context,
        initialScope
      )) {
        return;
      }
      if (isFragmentHasLessThanTwoChildren(node) && !isFragmentWithOnlyTextAndIsNotChild(node) && !(isFragmentWithSingleExpression(node))) {
        context.report({
          messageId: "NO_USELESS_FRAGMENT",
          node
        });
      }
      if (node.parent.type === ast.NodeType.JSXElement && jsx.isJSXElementOfBuiltinComponent(node.parent)) {
        context.report({
          messageId: "NO_USELESS_FRAGMENT_IN_BUILT_IN",
          node
        });
      }
    }
    return {
      JSXElement(node) {
        if (jsx.isFragmentElement(node, reactPragma, fragmentPragma)) {
          checkNode(node);
        }
      },
      JSXFragment: checkNode
    };
  },
  defaultOptions: []
});
var RULE_NAME38 = "prefer-destructuring-assignment";
function isMemberExpressionWithObjectName(node) {
  return node.object.type === ast.NodeType.Identifier && "name" in node.object;
}
var prefer_destructuring_assignment_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "enforce using destructuring assignment in component props and context",
      recommended: "recommended"
    },
    messages: {
      PREFER_DESTRUCTURING_ASSIGNMENT: "Prefer destructuring assignment."
    },
    schema: []
  },
  name: RULE_NAME38,
  create(context) {
    const { ctx, listeners } = core.useComponentCollector(context);
    const memberExpressionWithNames = [];
    return {
      ...listeners,
      MemberExpression(node) {
        if (isMemberExpressionWithObjectName(node)) {
          const scope = context.sourceCode.getScope(node);
          memberExpressionWithNames.push([scope, node]);
        }
      },
      "Program:exit"(node) {
        const components = Array.from(ctx.getAllComponents(node).values());
        function isFunctionComponent(block) {
          if (!ast.isFunction(block))
            return false;
          const maybeId = ast.getFunctionIdentifier(block);
          return Option_exports.isSome(maybeId) && core.isComponentName(maybeId.value.name) && components.some((component) => component.node === block);
        }
        for (const [scope, memberExpression] of memberExpressionWithNames) {
          const scopeRef = MutableRef_exports.make(scope);
          const isComponentRef = MutableRef_exports.make(isFunctionComponent(scope.block));
          while (!MutableRef_exports.get(isComponentRef) && MutableRef_exports.get(scopeRef).upper && MutableRef_exports.get(scopeRef).upper !== MutableRef_exports.get(scopeRef)) {
            MutableRef_exports.set(scopeRef, MutableRef_exports.get(scopeRef).upper);
            MutableRef_exports.set(isComponentRef, isFunctionComponent(MutableRef_exports.get(scopeRef).block));
          }
          if (!MutableRef_exports.get(isComponentRef))
            continue;
          const component = MutableRef_exports.get(scopeRef).block;
          if (!("params" in component))
            continue;
          const [props, ctx2] = component.params;
          const isMatch = a({ name: memberExpression.object.name });
          if (isMatch(props)) {
            context.report({
              messageId: "PREFER_DESTRUCTURING_ASSIGNMENT",
              node: memberExpression
            });
          }
          if (isMatch(ctx2)) {
            context.report({
              messageId: "PREFER_DESTRUCTURING_ASSIGNMENT",
              node: memberExpression
            });
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME39 = "prefer-shorthand-boolean";
var prefer_shorthand_boolean_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "enforce boolean attributes notation in JSX",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      PREFER_SHORTHAND_BOOLEAN: "Prefer shorthand boolean attributes."
    },
    schema: []
  },
  name: RULE_NAME39,
  create(context) {
    return {
      JSXAttribute(node) {
        const { value } = node;
        const propName = jsx.getPropName(node);
        const hasValueTrue = value?.type === ast.NodeType.JSXExpressionContainer && value.expression.type === ast.NodeType.Literal && value.expression.value === true;
        if (!hasValueTrue)
          return;
        context.report({
          data: {
            propName
          },
          messageId: "PREFER_SHORTHAND_BOOLEAN",
          node
        });
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME40 = "prefer-shorthand-fragment";
var prefer_shorthand_fragment_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "enforce using fragment syntax instead of Fragment component",
      recommended: "recommended",
      requiresTypeChecking: false
    },
    messages: {
      PREFER_SHORTHAND_FRAGMENT: "Prefer shorthand fragment syntax instead of '{{reactPragma}}.{{fragmentPragma}}'."
    },
    schema: []
  },
  name: RULE_NAME40,
  create(context) {
    const reactPragma = jsx.getPragmaFromContext(context);
    const fragmentPragma = jsx.getFragmentFromContext(context);
    return {
      JSXElement(node) {
        if (jsx.isFragmentElement(node, reactPragma, fragmentPragma)) {
          const hasAttributes = node.openingElement.attributes.length > 0;
          if (hasAttributes)
            return;
          context.report({
            data: {
              fragmentPragma,
              reactPragma
            },
            messageId: "PREFER_SHORTHAND_FRAGMENT",
            node
          });
        }
      }
    };
  },
  defaultOptions: []
});

// src/index.ts
var meta = {
  name,
  version
};
var rules = {
  "ensure-forward-ref-using-ref": ensure_forward_ref_using_ref_default,
  "no-access-state-in-setstate": no_access_state_in_setstate_default,
  "no-array-index-key": no_array_index_key_default,
  "no-children-count": no_children_count_default,
  "no-children-for-each": no_children_for_each_default,
  "no-children-map": no_children_map_default,
  "no-children-only": no_children_only_default,
  "no-children-prop": no_children_prop_default,
  "no-children-to-array": no_children_to_array_default,
  "no-class-component": no_class_component_default,
  "no-clone-element": no_clone_element_default,
  "no-comment-textnodes": no_comment_textnodes_default,
  "no-complicated-conditional-rendering": no_complicated_conditional_rendering_default,
  "no-component-will-mount": no_component_will_mount_default,
  "no-component-will-receive-props": no_component_will_receive_props_default,
  "no-component-will-update": no_component_will_update_default,
  "no-create-ref": no_create_ref_default,
  "no-direct-mutation-state": no_direct_mutation_state_default,
  "no-duplicate-key": no_duplicate_key_default,
  "no-implicit-key": no_implicit_key_default,
  "no-leaked-conditional-rendering": no_leaked_conditional_rendering_default,
  "no-missing-component-display-name": no_missing_component_display_name_default,
  "no-missing-key": no_missing_key_default,
  "no-nested-components": no_nested_components_default,
  "no-redundant-should-component-update": no_redundant_should_component_update_default,
  "no-set-state-in-component-did-mount": no_set_state_in_component_did_mount_default,
  "no-set-state-in-component-did-update": no_set_state_in_component_did_update_default,
  "no-set-state-in-component-will-update": no_set_state_in_component_will_update_default,
  "no-string-refs": no_string_refs_default,
  "no-unsafe-component-will-mount": no_unsafe_component_will_mount_default,
  "no-unsafe-component-will-receive-props": no_unsafe_component_will_receive_props_default,
  "no-unsafe-component-will-update": no_unsafe_component_will_update_default,
  "no-unstable-context-value": no_unstable_context_value_default,
  "no-unstable-default-props": no_unstable_default_props_default,
  "no-unused-class-component-members": no_unused_class_component_members_default,
  "no-unused-state": no_unused_state_default,
  "no-useless-fragment": no_useless_fragment_default,
  "prefer-destructuring-assignment": prefer_destructuring_assignment_default,
  "prefer-shorthand-boolean": prefer_shorthand_boolean_default,
  "prefer-shorthand-fragment": prefer_shorthand_fragment_default
};

exports.meta = meta;
exports.rules = rules;
